/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats;

import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.PortalEvent;
import org.jasig.portal.stats.item.write.IntervalAwareItemWriter;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;
import org.springframework.batch.repeat.ExitStatus;
import org.springframework.batch.repeat.RepeatContext;
import org.springframework.batch.repeat.policy.DefaultResultCompletionPolicy;

/**
 * A Job Completion policy implementation that triggers flushing when time interval boundries are encountered by the
 * events being processed. Syncing flushing with interval boundries helps reduce writes to the database. 
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class IntervalFlushingCompletionPolicy extends DefaultResultCompletionPolicy implements IntervalAwareItemWriter<PortalEvent> {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private boolean needsFlush = false;

    /**
     * Reset the flag.
     * 
     * @see org.springframework.batch.repeat.CompletionPolicy#start(RepeatContext)
     */
    @Override
    public RepeatContext start(RepeatContext context) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Start RepeatContext");
        }
        this.needsFlush = false;
        return super.start(context);
    }

    /**
     * Terminate if the chunk size has been reached, or the result is null.
     * 
     * @see org.springframework.batch.repeat.CompletionPolicy#isComplete(RepeatContext,
     * ExitStatus)
     * @throws RuntimeException (normally terminating the batch) if the result is
     * itself an exception.
     */
    @Override
    public boolean isComplete(RepeatContext context, ExitStatus result) {
        final boolean complete = super.isComplete(context, result) || this.needsFlush;
        if (complete && this.logger.isDebugEnabled()) {
            this.logger.debug("isComplete(" + context + ", " + result + ")");
        }
        return complete;
    }

    /**
     * Terminate if the chunk size has been reached.
     * 
     * @see org.springframework.batch.repeat.CompletionPolicy#isComplete(RepeatContext)
     */
    @Override
    public boolean isComplete(RepeatContext context) {
        if (this.needsFlush && this.logger.isDebugEnabled()) {
            this.logger.debug("isComplete(" + context + ")");
        }
        return this.needsFlush;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, java.util.Map)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Set Needs Flush");
        }
        this.needsFlush = true;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handles(org.jasig.portal.stats.Interval)
     */
    public boolean handles(Interval interval) {
        return true;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#supports(org.jasig.portal.stats.events.EventType)
     */
    public boolean supports(EventType eventType) {
        return true;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        //Ignore
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException {
        //Ignore
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doWrite(org.jasig.portal.stats.events.PortalEvent)
     */
    public void doWrite(PortalEvent item) throws Exception {
        //Ignore
    }
}
