/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.stats.dao.IntervalHelperDao;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;

/**
 * Determines the correct query or queries to run for a specified Interval and Date to find the start and end dates
 * of the interval.
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class IntervalHelperImpl implements IntervalHelper {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private final String yearParam;
    private final String monthParam;
    private final String dayParam;
    private final String hourParam;
    private final String minuteParam;
    
    private final String queryForYearStart;
    private final String queryForAcadTermStart;
    private final String queryForAcadTermEnd;
    private final String queryForCalQtrStart;
    private final String queryForMonthStart;
    private final String queryForWeekStart;
    private final String queryForDayStart;
    private final String queryForHourStart;
    private final String queryForFiveMinuteStart;
    private final String queryForMinuteStart;
    
    private IntervalHelperDao intervalHelperDao;
    
    public IntervalHelperImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/IntervalHelperImplSql.xml", this.getClass().getClassLoader());
        
        this.yearParam = sqlLoader.getSql("yearParam");
        this.monthParam = sqlLoader.getSql("monthParam");
        this.dayParam = sqlLoader.getSql("dayParam");
        this.hourParam = sqlLoader.getSql("hourParam");
        this.minuteParam = sqlLoader.getSql("minuteParam");
        
        this.queryForYearStart = sqlLoader.getSql("queryForYearStart");
        this.queryForAcadTermStart = sqlLoader.getSql("queryForAcadTermStart");
        this.queryForAcadTermEnd = sqlLoader.getSql("queryForAcadTermEnd");
        this.queryForCalQtrStart = sqlLoader.getSql("queryForCalQtrStart");
        this.queryForMonthStart = sqlLoader.getSql("queryForMonthStart");
        this.queryForWeekStart = sqlLoader.getSql("queryForWeekStart");
        this.queryForDayStart = sqlLoader.getSql("queryForDayStart");
        this.queryForHourStart = sqlLoader.getSql("queryForHourStart");
        this.queryForFiveMinuteStart = sqlLoader.getSql("queryForFiveMinuteStart");
        this.queryForMinuteStart = sqlLoader.getSql("queryForMinuteStart");
    }
    
    public IntervalHelperDao getIntervalHelperDao() {
        return this.intervalHelperDao;
    }
    /**
     * The IntervalHelperDao used to get interval information from the database.
     */
    public void setIntervalHelperDao(IntervalHelperDao intervalHelperDao) {
        this.intervalHelperDao = intervalHelperDao;
    }



    /* (non-Javadoc)
     * @see org.jasig.portal.stats.IntervalHelper#getIntervalDates(org.jasig.portal.stats.Interval, java.util.Date)
     */
    public IntervalInfo getIntervalDates(Interval interval, Date date) {
        final Date start;
        final Date end;
        final int startDateKey;
        final int startTimeKey;
        
        final Calendar now = Calendar.getInstance();
        now.clear();
        now.setTime(date);

        final int year = now.get(Calendar.YEAR);
        final int month = now.get(Calendar.MONTH) + 1;
        final int day = now.get(Calendar.DAY_OF_MONTH);
        final int hour = now.get(Calendar.HOUR_OF_DAY);
        final int minute = now.get(Calendar.MINUTE);
        
        final Map<String, Object> dateQueryParams = new HashMap<String, Object>();
        dateQueryParams.put(this.yearParam, year);
        dateQueryParams.put(this.monthParam, month);
        dateQueryParams.put(this.dayParam, day);
        
        final Map<String, Object> timeQueryParams = new HashMap<String, Object>();
        timeQueryParams.put(this.hourParam, hour);
        timeQueryParams.put(this.minuteParam, minute);
        
        //Determine the query for the date (y/m/d) part of the interval start 
        final String dateQuery;
        switch (interval) {
            case YEAR: {
                dateQuery = this.queryForYearStart;
            } break;

            case ACADEMIC_TERM: {
                dateQuery = this.queryForAcadTermStart;
            } break;
            
            case CALENDAR_QUARTER: {
                dateQuery = this.queryForCalQtrStart;
            } break;
            
            case MONTH: {
                dateQuery = this.queryForMonthStart;
            } break;
            
            case WEEK: {
                dateQuery = this.queryForWeekStart;
            } break;
            
            case DAY:
            case HOUR:
            case FIVE_MINUTE:
            case MINUTE: {
                dateQuery = this.queryForDayStart;
            } break;
            
            default: {
                throw new IllegalArgumentException("Specified Interval " + interval + " is not supported");
            }
        }
        
        
        
        //Run the chosen date query and start building the startDate calendar
        final Map<String, Number> dateResults = this.intervalHelperDao.queryForMap(dateQuery, dateQueryParams);
        startDateKey = dateResults.get("DATE_DIMENSION_KEY").intValue();
        final Calendar baseStartDate = this.getDate(dateResults);

        //Build the time portion of the startDate calendar and setup the end date
        switch (interval) {
            case YEAR: {
                startTimeKey = 1;
                start = baseStartDate.getTime();
                end = DateUtils.addYears(start, 1);
            } break;

            //Academic terms are arbitrary and require an additional database query to determine the end date
            case ACADEMIC_TERM: {
                startTimeKey = 1;
                start = baseStartDate.getTime();
                
                final Map<String, Number> endResults = this.intervalHelperDao.queryForMap(this.queryForAcadTermEnd, dateQueryParams);
                final Calendar baseEndDate = this.getDate(endResults);
                end = DateUtils.addDays(baseEndDate.getTime(), 1);
            } break;
            
            case CALENDAR_QUARTER: {
                startTimeKey = 1;
                start = baseStartDate.getTime();
                end = DateUtils.addMonths(start, 3);
            } break;
            
            case MONTH: {
                startTimeKey = 1;
                start = baseStartDate.getTime();
                end = DateUtils.addMonths(start, 1);
            } break;
            
            case WEEK: {
                startTimeKey = 1;
                start = baseStartDate.getTime();
                end = DateUtils.addDays(start, 7);
            } break;
            
            case DAY: {
                startTimeKey = 1;
                start = baseStartDate.getTime();
                end = DateUtils.addDays(start, 1);
            } break;
            
            case HOUR: {
                final Map<String, Number> timeResults = this.intervalHelperDao.queryForMap(this.queryForHourStart, timeQueryParams);
                startTimeKey = timeResults.get("TIME_DIMENSION_KEY").intValue();
                
                baseStartDate.set(Calendar.HOUR, timeResults.get("HOUR").intValue());
                baseStartDate.set(Calendar.MINUTE, timeResults.get("MINUTE").intValue());
                start = baseStartDate.getTime();

                end = DateUtils.addHours(start, 1);
            } break;
            
            case FIVE_MINUTE: {
                final Map<String, Number> timeResults = this.intervalHelperDao.queryForMap(this.queryForFiveMinuteStart, timeQueryParams);
                startTimeKey = timeResults.get("TIME_DIMENSION_KEY").intValue();
                
                baseStartDate.set(Calendar.HOUR, timeResults.get("HOUR").intValue());
                baseStartDate.set(Calendar.MINUTE, timeResults.get("MINUTE").intValue());
                start = baseStartDate.getTime();

                end = DateUtils.addMinutes(start, 5);
            } break;
            
            case MINUTE: {
                final Map<String, Number> timeResults = this.intervalHelperDao.queryForMap(this.queryForMinuteStart, timeQueryParams);
                startTimeKey = timeResults.get("TIME_DIMENSION_KEY").intValue();
                
                baseStartDate.set(Calendar.HOUR, timeResults.get("HOUR").intValue());
                baseStartDate.set(Calendar.MINUTE, timeResults.get("MINUTE").intValue());
                start = baseStartDate.getTime();

                end = DateUtils.addMinutes(start, 1);
            } break;
            
            default: {
                throw new IllegalArgumentException("Specified Interval " + interval + " is not supported");
            }
        }
        
        final IntervalInfo intervalInfo = new IntervalInfo(start, end, startDateKey, startTimeKey);
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Calculated " + intervalInfo + " for interval=" + interval + " and date=" + date);
        }
        
        return intervalInfo;
    }

    /**
     * Grabs the year/month/day from the results map and creates a Date for the values
     */
    protected Calendar getDate(final Map<String, Number> results) {
        final Calendar cal = Calendar.getInstance();
        cal.clear();
        
        final int startYear = results.get("CALENDAR_YEAR_NUMBER").intValue();
        final int startMonth = results.get("CALENDAR_MONTH_NUMBER").intValue();
        final int startDay = results.get("CALENDAR_DAY_NUMBER").intValue();
        
        cal.set(startYear, startMonth - 1, startDay);
        
        return cal;
    }
}
