/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats;

import java.util.Date;

import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.stats.dao.JobDateStore;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobExecutionListener;
import org.springframework.batch.core.JobInstance;
import org.springframework.batch.core.JobParameters;

/**
 * Listens for the end of a step execution and stores the last processed event date specified in the ExecutionContext
 * via {@link JobDateStore}
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class JobCompletionEventListener implements JobExecutionListener {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private JobDateStore jobDateStore;
    
    public JobDateStore getLastJobEventDao() {
        return jobDateStore;
    }
    /**
     * Data source for the last processed event for a job
     */
    public void setLastJobEventDao(JobDateStore jobDateStore) {
        this.jobDateStore = jobDateStore;
    }
    
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#afterJob(org.springframework.batch.core.JobExecution)
     */
    public void afterJob(JobExecution jobExecution) {
        final JobInstance jobInstance = jobExecution.getJobInstance();
        final JobParameters jobParameters = jobInstance.getJobParameters();
        final Date startDate = jobParameters.getDate(JobDateStore.STORED_DATE);
        
        final Date lastEvent = new Date();
        
        final Date endDate;
        if (startDate != null && !lastEvent.after(startDate)) {
            //Fail safe and make sure that we step forward at least a second between runs
            this.logger.warn("Adding 1 second to " + startDate + "' to avoid failure to re-run due to duplicate paramters");
            endDate = DateUtils.addSeconds(startDate, 1);
        }
        else {
            endDate = lastEvent;
        }
        
        final String jobName = jobInstance.getJobName();
        this.jobDateStore.setStoredDate(jobName, endDate);
        
        if (this.logger.isInfoEnabled()) {
            this.logger.info("Stored end date '" + endDate + "' in JobDateStore");
        }
    }
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#beforeJob(org.springframework.batch.core.JobExecution)
     */
    public void beforeJob(JobExecution jobExecution) {
        //Ignore
    }
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#onError(org.springframework.batch.core.JobExecution, java.lang.Throwable)
     */
    public void onError(JobExecution jobExecution, Throwable e) {
        //Ignore
    }
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#onInterrupt(org.springframework.batch.core.JobExecution)
     */
    public void onInterrupt(JobExecution jobExecution) {
        //Ignore
    }
}
