/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats;

import java.util.Date;

import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.stats.dao.JobDateStore;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobInstance;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.StepExecutionListener;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.repeat.ExitStatus;

/**
 * Listens for the end of a step execution and stores the last processed event date specified in the ExecutionContext
 * via {@link JobDateStore}
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class LastPortalEventListener implements StepExecutionListener {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private JobDateStore jobDateStore;
    
    public JobDateStore getLastJobEventDao() {
        return jobDateStore;
    }
    /**
     * Data source for the last processed event for a job
     */
    public void setLastJobEventDao(JobDateStore jobDateStore) {
        this.jobDateStore = jobDateStore;
    }
    
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.StepExecutionListener#afterStep(org.springframework.batch.core.StepExecution)
     */
    public ExitStatus afterStep(StepExecution stepExecution) {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("afterStep()");
        }
        
        final JobParameters jobParameters = stepExecution.getJobParameters();
        final Date startDate = jobParameters.getDate(JobDateStore.STORED_DATE);
        
        final ExecutionContext executionContext = stepExecution.getExecutionContext();
        final Date lastEvent = (Date)executionContext.get(JobDateStore.STORED_DATE);
        
        final Date endDate;
        if (lastEvent == null || (startDate != null && !lastEvent.after(startDate))) {
            //Fail safe and make sure that we step forward at least a second between runs
            this.logger.warn("Adding 1 second to " + startDate + "' to avoid failure to re-run due to duplicate paramters");
            endDate = DateUtils.addSeconds(startDate, 1);
        }
        else {
            endDate = lastEvent;
        }
        
        final JobExecution jobExecution = stepExecution.getJobExecution();
        final JobInstance jobInstance = jobExecution.getJobInstance();
        final String jobName = jobInstance.getJobName();
        
        final long storeStart = System.currentTimeMillis();
        this.jobDateStore.setStoredDate(jobName, endDate);
        final long storeTime = System.currentTimeMillis() - storeStart;
        
        if (this.logger.isInfoEnabled()) {
            this.logger.info("Stored last processed date '" + endDate + "' in JobDateStore in " + storeTime + "ms");
        }
        
        return null;
    }
    /* (non-Javadoc)
     * @see org.springframework.batch.core.StepExecutionListener#beforeStep(org.springframework.batch.core.StepExecution)
     */
    public void beforeStep(StepExecution stepExecution) {
    }
    /* (non-Javadoc)
     * @see org.springframework.batch.core.StepExecutionListener#onErrorInStep(org.springframework.batch.core.StepExecution, java.lang.Throwable)
     */
    public ExitStatus onErrorInStep(StepExecution stepExecution, Throwable e) {
        return null;
    }
}
