/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.stats.dao.JobDateStore;
import org.jasig.portal.stats.quartz.JobParametersProvider;
import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.JobParametersBuilder;

/**
 * Uses {@link JobDateStore} to provide the last processed event, as stored in the DAO. By default the name of the
 * {@link Job} passed to {@link #getJobParameters(Job)} is used with the DAO. A targetJobName can be specified as well
 * and then that explicit value is used.
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class StoredJobDateParameterProvider implements JobParametersProvider {
    private static final DateFormat DATE_PARAMETER_FORMAT = new SimpleDateFormat("EEE MMM dd HH:mm:ss.SSS zzz yyyy", Locale.US);
    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private String targetJobName;
    private JobDateStore jobDateStore;
    
    public JobDateStore getLastJobEventDao() {
        return jobDateStore;
    }
    /**
     * Data source for the last processed event for a job
     */
    public void setLastJobEventDao(JobDateStore jobDateStore) {
        this.jobDateStore = jobDateStore;
    }
    
    public String getTargetJobName() {
        return targetJobName;
    }
    /**
     * Name of the job to get the last processed date for
     */
    public void setTargetJobName(String targetJobName) {
        this.targetJobName = targetJobName;
    }


    /* (non-Javadoc)
     * @see org.jasig.portal.stats.quartz.JobParametersProvider#getJobParameters(org.springframework.batch.core.Job)
     */
    public JobParameters getJobParameters(Job job) {
        final JobParametersBuilder parametersBuilder = new JobParametersBuilder();
        
        final String jobName;
        if (this.targetJobName != null) {
            jobName = this.targetJobName;
        }
        else {
            jobName = job.getName();
        }
        
        final Date lastEvent = this.jobDateStore.getStoredDate(jobName);
        if (lastEvent != null) {
            parametersBuilder.addDate(JobDateStore.STORED_DATE, lastEvent);
            
            //Also add date as a String to include 100% of the data
            final String formattedDate;
            synchronized (DATE_PARAMETER_FORMAT) {
                //Logic cloned from java.util.Date
                DATE_PARAMETER_FORMAT.setTimeZone(TimeZone.getDefault());
                formattedDate = DATE_PARAMETER_FORMAT.format(lastEvent);
            }
            parametersBuilder.addString(JobDateStore.STORED_DATE + "_STR", formattedDate);
        }

        final JobParameters jobParameters = parametersBuilder.toJobParameters();
        if (this.logger.isInfoEnabled()) {
            this.logger.info("Loaded JobParameters " + jobParameters + " for job '" + job.getName() + "' using jobName '" + jobName + "'");
        }
        
        return jobParameters;
    }
}
