/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.ChannelInfo;
import org.springframework.batch.item.database.support.DefaultDataFieldMaxValueIncrementerFactory;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

/**
 * Uses a lookup table and {@link DataFieldMaxValueIncrementer} to get/create stats channel keys. 
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class ChannelStatsKeyDaoImpl extends SimpleJdbcDaoSupport implements ChannelStatsKeyDao {
    private final ParameterizedRowMapper<Number> mapper;
    
    private final String fnameParam;
    private final String statsKeyParam;
    private final String titleParam;
    private final String nameParam;
    private final String descParam;
    
    private final String channelKeySequenceName;
    private final String channelKeyForFnameQuery;
    private final String insertChannelInfo;
    
    private TransactionTemplate transactionTemplate;
    private DataFieldMaxValueIncrementer channelKeyIncrementer;
    
    private String databaseType;
    private PlatformTransactionManager transactionManager;
    
    public ChannelStatsKeyDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/ChannelStatsKeyImplSql.xml", this.getClass().getClassLoader());

        this.fnameParam = sqlLoader.getSql("fnameParam");
        this.statsKeyParam = sqlLoader.getSql("statsKeyParam");
        this.titleParam = sqlLoader.getSql("titleParam");
        this.nameParam = sqlLoader.getSql("nameParam");
        this.descParam = sqlLoader.getSql("descParam");
        
        this.channelKeyForFnameQuery = sqlLoader.getSql("channelKeyForFnameQuery");
        this.channelKeySequenceName = sqlLoader.getSql("channelKeySequenceName");
        this.insertChannelInfo = sqlLoader.getSql("insertChannelInfo");
        
        this.mapper = ParameterizedSingleColumnRowMapper.newInstance(Number.class);
    }
    
    public PlatformTransactionManager getTransactionManager() {
        return transactionManager;
    }
    /**
     * Transaction manager to use for creating channel stats key entries
     */
    public void setTransactionManager(PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public String getDatabaseType() {
        return databaseType;
    }
    /**
     * The type of database being used. From {@link DefaultDataFieldMaxValueIncrementerFactory}
     */
    public void setDatabaseType(String databaseType) {
        this.databaseType = databaseType;
    }

    /* (non-Javadoc)
     * @see org.springframework.dao.support.DaoSupport#initDao()
     */
    @Override
    protected void initDao() throws Exception {
        this.transactionTemplate = new TransactionTemplate(this.transactionManager);
        this.transactionTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
        this.transactionTemplate.afterPropertiesSet();
        
        
        final DefaultDataFieldMaxValueIncrementerFactory incrementerFactory = new DefaultDataFieldMaxValueIncrementerFactory(this.getDataSource());
        Assert.isTrue(incrementerFactory.isSupportedIncrementerType(this.databaseType), "'" + this.databaseType
                + "' is an unsupported database type.  The supported database types are "
                + StringUtils.arrayToCommaDelimitedString(incrementerFactory.getSupportedIncrementerTypes()));
        
        this.channelKeyIncrementer = incrementerFactory.getIncrementer(this.databaseType, this.channelKeySequenceName);
    }

    public long getChannelStatsKey(ChannelInfo channelInfo) {
        final SimpleJdbcTemplate simpleJdbcTemplate = this.getSimpleJdbcTemplate();
        
        final String fname = channelInfo.getFname();
        final Map<String, String> queryParams = Collections.singletonMap(this.fnameParam, fname);
        final List<Number> results = simpleJdbcTemplate.query(this.channelKeyForFnameQuery, this.mapper, queryParams);

        final long channelKey;
        if (results.size() == 0) {
            channelKey = (Long)this.transactionTemplate.execute(new InsertChannelInfoCallback(channelInfo));
        }
        else {
            channelKey = results.get(0).longValue();
        }
        
        return channelKey;
    }
    
    private final class InsertChannelInfoCallback implements TransactionCallback {
        private final ChannelInfo channelInfo;

        public InsertChannelInfoCallback(ChannelInfo channelInfo) {
            this.channelInfo = channelInfo;
        }

        public Object doInTransaction(TransactionStatus status) {
            final long nextChannelKey = ChannelStatsKeyDaoImpl.this.channelKeyIncrementer.nextLongValue();
            
            final Map<String, Object> insertParams = new HashMap<String, Object>();
            insertParams.put(statsKeyParam, nextChannelKey);
            insertParams.put(fnameParam, this.channelInfo.getFname());
            insertParams.put(titleParam, this.channelInfo.getTitle());
            insertParams.put(nameParam, this.channelInfo.getName());
            insertParams.put(descParam, this.channelInfo.getDescription());
            
            ChannelStatsKeyDaoImpl.this.getSimpleJdbcTemplate().update(ChannelStatsKeyDaoImpl.this.insertChannelInfo, insertParams);
            
            return nextChannelKey;
        }
    }
}
