/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.Interval;
import org.springframework.batch.item.database.support.DefaultDataFieldMaxValueIncrementerFactory;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import edu.emory.mathcs.backport.java.util.Collections;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class IntervalStatsKeyDaoImpl extends SimpleJdbcDaoSupport implements IntervalStatsKeyDao {
    private final ParameterizedRowMapper<Number> mapper;

    private final String keyParam;
    private final String nameParam;
    
    private final String intervalKeySequenceName;
    private final String intervalKeyForNameQuery;
    private final String insertIntervalKey;
    
    private TransactionTemplate transactionTemplate;
    private DataFieldMaxValueIncrementer intervalKeyIncrementer;
    
    private String databaseType;
    private PlatformTransactionManager transactionManager;
    
    public IntervalStatsKeyDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/IntervalStatsKeyImplSql.xml", this.getClass().getClassLoader());

        this.keyParam = sqlLoader.getSql("keyParam");
        this.nameParam = sqlLoader.getSql("nameParam");
        
        this.intervalKeySequenceName = sqlLoader.getSql("intervalKeySequenceName");
        this.intervalKeyForNameQuery = sqlLoader.getSql("intervalKeyForNameQuery");
        this.insertIntervalKey = sqlLoader.getSql("insertIntervalKey");
        
        this.mapper = ParameterizedSingleColumnRowMapper.newInstance(Number.class);
    }
    
    public PlatformTransactionManager getTransactionManager() {
        return transactionManager;
    }
    /**
     * Transaction manager to use for creating channel stats key entries
     */
    public void setTransactionManager(PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public String getDatabaseType() {
        return databaseType;
    }
    /**
     * The type of database being used. From {@link DefaultDataFieldMaxValueIncrementerFactory}
     */
    public void setDatabaseType(String databaseType) {
        this.databaseType = databaseType;
    }

    /* (non-Javadoc)
     * @see org.springframework.dao.support.DaoSupport#initDao()
     */
    @Override
    protected void initDao() throws Exception {
        this.transactionTemplate = new TransactionTemplate(this.transactionManager);
        this.transactionTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
        this.transactionTemplate.afterPropertiesSet();
        
        
        final DefaultDataFieldMaxValueIncrementerFactory incrementerFactory = new DefaultDataFieldMaxValueIncrementerFactory(this.getDataSource());
        Assert.isTrue(incrementerFactory.isSupportedIncrementerType(this.databaseType), "'" + this.databaseType
                + "' is an unsupported database type.  The supported database types are "
                + StringUtils.arrayToCommaDelimitedString(incrementerFactory.getSupportedIncrementerTypes()));
        
        this.intervalKeyIncrementer = incrementerFactory.getIncrementer(this.databaseType, this.intervalKeySequenceName);
    }

    
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.IntervalStatsKeyDao#getIntervalKey(org.jasig.portal.stats.Interval)
     */
    public long getIntervalKey(Interval interval) {
        final SimpleJdbcTemplate simpleJdbcTemplate = this.getSimpleJdbcTemplate();
        
        final Map<?, ?> queryParams = Collections.singletonMap(this.nameParam, interval.name());
        final List<Number> results = simpleJdbcTemplate.query(this.intervalKeyForNameQuery, this.mapper, queryParams);

        final long intervalKey;
        if (results.size() == 0) {
            intervalKey = (Long)this.transactionTemplate.execute(new InsertIntervalCallback(interval));
        }
        else {
            intervalKey = results.get(0).longValue();
        }
        
        return intervalKey;
    }
    
    private final class InsertIntervalCallback implements TransactionCallback {
        private final Interval interval;
        
        public InsertIntervalCallback(Interval interval) {
            this.interval = interval;
        }

        public Object doInTransaction(TransactionStatus status) {
            final long nextIntervalKey = intervalKeyIncrementer.nextLongValue();
            
            final Map<String, Object> insertParams = new HashMap<String, Object>();
            insertParams.put(keyParam, nextIntervalKey);
            insertParams.put(nameParam, interval.name());
            
            final SimpleJdbcTemplate simpleJdbcTemplate = getSimpleJdbcTemplate();
            simpleJdbcTemplate.update(insertIntervalKey, insertParams);
            
            return nextIntervalKey;
        }
    }
}
