/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao;

import java.sql.Timestamp;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jasig.portal.stats.SqlLoader;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.transaction.annotation.Transactional;

/**
 * Helper to track the last processed event for a job
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class JobDateStoreImpl extends SimpleJdbcDaoSupport implements JobDateStore {
    private static final String LAST_JOB_EVENT_DAO_SQL_XML = "org/jasig/portal/stats/dao/JobDateStoreImplSql.xml";
    
    private final ParameterizedRowMapper<Timestamp> mapper;

    private final String nameParam;
    private final String lastDateParam;
    
    private final String lastEventDateForJobQuery;
    private final String insertEventDateForJob;
    private final String updateEventDateForJob;
    
    public JobDateStoreImpl() {
        final SqlLoader sqlLoader = new SqlLoader(LAST_JOB_EVENT_DAO_SQL_XML, this.getClass().getClassLoader());
        
        this.nameParam = sqlLoader.getSql("nameParam");
        this.lastDateParam = sqlLoader.getSql("lastDateParam");
        
        this.lastEventDateForJobQuery = sqlLoader.getSql("lastEventDateForJobQuery");
        this.insertEventDateForJob = sqlLoader.getSql("insertEventDateForJob");
        this.updateEventDateForJob = sqlLoader.getSql("updateEventDateForJob");
        
        this.mapper = ParameterizedSingleColumnRowMapper.newInstance(Timestamp.class);
    }
    
    public Date getStoredDate(String jobName) {
        final Map<?, ?> queryParams = Collections.singletonMap(this.nameParam, jobName);
        
        final List<Timestamp> results = this.getSimpleJdbcTemplate().query(this.lastEventDateForJobQuery, this.mapper, queryParams);
        final Timestamp lastEventDate = (Timestamp)DataAccessUtils.singleResult(results);
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Loaded lastEvent '" + lastEventDate + "' for jobName '" + jobName + "'");
        }
        
        if (lastEventDate == null) {
            return null;
        }
        return new Date(lastEventDate.getTime());
    }
    
    @Transactional
    public void setStoredDate(String jobName, Date lastEventDate) {
        final Map<String, Object> updateParams = new HashMap<String, Object>();
        updateParams.put(this.nameParam, jobName);
        updateParams.put(this.lastDateParam, lastEventDate);
        
        final SimpleJdbcTemplate simpleJdbcTemplate = this.getSimpleJdbcTemplate();
        final int rowsUpdated = simpleJdbcTemplate.update(this.updateEventDateForJob, updateParams);
        if (rowsUpdated == 0) {
            simpleJdbcTemplate.update(this.insertEventDateForJob, updateParams);
        }
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Stored lastEvent '" + lastEventDate + "' for jobName '" + jobName + "'");
        }
    }
}
