/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.TabInfo;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class TabInfoDaoImpl extends SimpleJdbcDaoSupport implements TabInfoDao {
    private final ParameterizedRowMapper<String> mapper;
    
    private final String userNameParam;
    private final String profileIdParam;
    private final String userIdParam;
    private final String layoutIdParam;
    private final String structIdParam;
    
    private final String tabNameForStructNodeQuery;
    private final String userLayoutInfoQuery;

    private TabStatsKeyDao tabStatsKeyDao;
    

    public TabInfoDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/TabInfoDaoImplSql.xml", this.getClass().getClassLoader());

        this.userNameParam = sqlLoader.getSql("userNameParam");
        this.profileIdParam = sqlLoader.getSql("profileIdParam");
        this.userIdParam = sqlLoader.getSql("userIdParam");
        this.layoutIdParam = sqlLoader.getSql("layoutIdParam");
        this.structIdParam = sqlLoader.getSql("structIdParam");
        
        this.tabNameForStructNodeQuery = sqlLoader.getSql("tabNameForStructNodeQuery");
        this.userLayoutInfoQuery = sqlLoader.getSql("userLayoutInfoQuery");
        
        this.mapper = ParameterizedSingleColumnRowMapper.newInstance(String.class);
    }
    

    public TabStatsKeyDao getTabStatsKeyDao() {
        return tabStatsKeyDao;
    }
    /**
     * DAO to get stats specific TabInfo keys from 
     */
    public void setTabStatsKeyDao(TabStatsKeyDao tabStatsKeyDao) {
        this.tabStatsKeyDao = tabStatsKeyDao;
    }
    
    
    /**
     * Gets the tab name for the specified layout node
     */
    public TabInfo getTabInfo(final long userId, final long layoutId, final long nodeId) {
        final Map<String, Object> queryParams = new HashMap<String, Object>();
        queryParams.put(this.userIdParam, userId);
        queryParams.put(this.layoutIdParam, layoutId);
        queryParams.put(this.structIdParam, nodeId);
        
        final String tabName = this.getSimpleJdbcTemplate().queryForObject(this.tabNameForStructNodeQuery, this.mapper, queryParams);
        
        final TabInfo tabInfo = new TabInfo();
        tabInfo.setName(tabName);
        
        final long tabStatsKey = this.tabStatsKeyDao.getTabStatsKey(tabInfo);
        tabInfo.setTabStatsKey(tabStatsKey);
        
        return tabInfo;
    }
    
    /**
     * Gets the user id and layout id for a username, profile and layout
     */
    public UserLayoutInfo getUserLayoutInfo(String username, int profileId) {
        final Map<String, Object> queryParams = new HashMap<String, Object>();
        queryParams.put(this.userNameParam, username);
        queryParams.put(this.profileIdParam, profileId);
        
        final Map<String, Object> userLayoutInfo = this.getSimpleJdbcTemplate().queryForMap(this.userLayoutInfoQuery, queryParams);
        final long userId = ((Number)userLayoutInfo.get("USER_ID")).longValue();
        final long layoutId = ((Number)userLayoutInfo.get("LAYOUT_ID")).longValue();
        
        return new UserLayoutInfo(userId, layoutId);
    }
    
    public static class UserLayoutInfo implements Serializable {
        private static final long serialVersionUID = 1L;
        
        public final long userId;
        public final long layoutId;

        public UserLayoutInfo(long userId, long layoutId) {
            this.userId = userId;
            this.layoutId = layoutId;
        }

        /**
         * @see java.lang.Object#equals(Object)
         */
        @Override
        public boolean equals(Object object) {
            if (object == this) {
                return true;
            }
            if (!(object instanceof UserLayoutInfo)) {
                return false;
            }
            UserLayoutInfo rhs = (UserLayoutInfo) object;
            return new EqualsBuilder()
                .append(this.userId, rhs.userId)
                .append(this.layoutId, rhs.layoutId)
                .isEquals();
        }

        /**
         * @see java.lang.Object#toString()
         */
        @Override
        public String toString() {
            return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
                .append("userId", this.userId)
                .append("layoutId", this.layoutId)
                .toString();
        }

        /**
         * @see java.lang.Object#hashCode()
         */
        @Override
        public int hashCode() {
            return new HashCodeBuilder(-801721637, -1502329325)
                .append(this.userId)
                .append(this.layoutId)
                .toHashCode();
        }
    }
}
