/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.TabInfo;
import org.springframework.batch.item.database.support.DefaultDataFieldMaxValueIncrementerFactory;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class TabStatsKeyDaoImpl extends SimpleJdbcDaoSupport implements TabStatsKeyDao {
    private final ParameterizedRowMapper<Number> mapper;

    private final String tabNameParam;
    private final String tabKeyParam;
    
    private final String tabKeySequenceName;
    private final String tabKeyForNameQuery;
    private final String insertTabInfo;
    
    private TransactionTemplate transactionTemplate;
    private DataFieldMaxValueIncrementer tabKeyIncrementer;
    
    private String databaseType;
    private PlatformTransactionManager transactionManager;
    
    public TabStatsKeyDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/TabStatsKeyImplSql.xml", this.getClass().getClassLoader());

        this.tabNameParam = sqlLoader.getSql("tabNameParam");
        this.tabKeyParam = sqlLoader.getSql("tabKeyParam");
        
        this.tabKeyForNameQuery = sqlLoader.getSql("tabKeyForNameQuery");
        this.tabKeySequenceName = sqlLoader.getSql("tabKeySequenceName");
        this.insertTabInfo = sqlLoader.getSql("insertTabInfo");
        
        this.mapper = ParameterizedSingleColumnRowMapper.newInstance(Number.class);
    }
    
    public PlatformTransactionManager getTransactionManager() {
        return transactionManager;
    }
    /**
     * Transaction manager to use for creating channel stats key entries
     */
    public void setTransactionManager(PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public String getDatabaseType() {
        return databaseType;
    }
    /**
     * The type of database being used. From {@link DefaultDataFieldMaxValueIncrementerFactory}
     */
    public void setDatabaseType(String databaseType) {
        this.databaseType = databaseType;
    }

    /* (non-Javadoc)
     * @see org.springframework.dao.support.DaoSupport#initDao()
     */
    @Override
    protected void initDao() throws Exception {
        this.transactionTemplate = new TransactionTemplate(this.transactionManager);
        this.transactionTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
        this.transactionTemplate.afterPropertiesSet();
        
        
        final DefaultDataFieldMaxValueIncrementerFactory incrementerFactory = new DefaultDataFieldMaxValueIncrementerFactory(this.getDataSource());
        Assert.isTrue(incrementerFactory.isSupportedIncrementerType(this.databaseType), "'" + this.databaseType
                + "' is an unsupported database type.  The supported database types are "
                + StringUtils.arrayToCommaDelimitedString(incrementerFactory.getSupportedIncrementerTypes()));
        
        this.tabKeyIncrementer = incrementerFactory.getIncrementer(this.databaseType, this.tabKeySequenceName);
    }

    public long getTabStatsKey(TabInfo tabInfo) {
        final SimpleJdbcTemplate simpleJdbcTemplate = this.getSimpleJdbcTemplate();
        
        final Map<?, ?> queryParams = Collections.singletonMap(this.tabNameParam, tabInfo.getName());
        final List<Number> results = simpleJdbcTemplate.query(this.tabKeyForNameQuery, this.mapper, queryParams);

        final long tabKey;
        if (results.size() == 0) {
            tabKey = (Long)this.transactionTemplate.execute(new InsertTabInfoCallback(tabInfo));
        }
        else {
            tabKey = results.get(0).longValue();
        }
        
        return tabKey;
    }
    
    private final class InsertTabInfoCallback implements TransactionCallback {
        private final TabInfo tabInfo;

        public InsertTabInfoCallback(TabInfo tabInfo) {
            this.tabInfo = tabInfo;
        }

        public Object doInTransaction(TransactionStatus status) {
            final long nextTabKey = tabKeyIncrementer.nextLongValue();
            final String tabName = this.tabInfo.getName();
            
            final Map<String, Object> insertParams = new HashMap<String, Object>();
            insertParams.put(tabNameParam, tabName);
            insertParams.put(tabKeyParam, nextTabKey);
            
            final SimpleJdbcTemplate simpleJdbcTemplate = getSimpleJdbcTemplate();
            simpleJdbcTemplate.update(TabStatsKeyDaoImpl.this.insertTabInfo, insertParams);
            
            return nextTabKey;
        }
    }
}
