/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao.aggr;

import java.text.DecimalFormat;
import java.util.Collection;
import java.util.Map;

import org.jasig.portal.stats.Average;
import org.jasig.portal.stats.dao.IntervalStatsKeyDao;
import org.jasig.portal.stats.om.Interval;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

/**
 * Base DAO that supports batching JDBC operations using a specified flush interval.
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class BatchingJdbcDaoSupport extends SimpleJdbcDaoSupport {
    private final static DecimalFormat AVG_FORMATTER = new DecimalFormat("0.00");
    
    private int batchSize = 25;
    private IntervalStatsKeyDao intervalStatsKeyDao;
    
    public int getBatchSize() {
        return this.batchSize;
    }
    /**
     * The number of sets of parameters to send in a single batch update
     */
    public void setBatchSize(int batchSize) {
        this.batchSize = batchSize;
    }
    public IntervalStatsKeyDao getIntervalStatsKeyDao() {
        return intervalStatsKeyDao;
    }
    /**
     * Used to look up the numeric database key for a given interval
     */
    public void setIntervalStatsKeyDao(IntervalStatsKeyDao intervalStatsKeyDao) {
        this.intervalStatsKeyDao = intervalStatsKeyDao;
    }

    /**
     * @see IntervalStatsKeyDao#getIntervalKey(Interval)
     */
    protected long getIntervalKey(final Interval interval) {
        return this.intervalStatsKeyDao.getIntervalKey(interval);
    }

    /**
     * Executes {@link SimpleJdbcTemplate#batchUpdate(String, Map[])} using the specified Collection of
     * parameter Maps. If there are more Maps than {@link #getBatchSize()} the underlying calls to {@link SimpleJdbcTemplate#batchUpdate(String, Map[])}
     * will be broken up with a maximum of {@link #getBatchSize()} parameter Maps per call.
     */
    public final int[] batchUpdate(String sql, Collection<Map<String, Object>> parameters) {
        int resultsIndex = 0;
        final int[] allResults = new int[parameters.size()];
        
        final Average batchAverage = new Average();
        
        int paramCount = 0;
        final SqlParameterSource[] batchArgsBuilder = new SqlParameterSource[this.batchSize];
        for (final Map<String, Object> parameterMap : parameters) {
            batchArgsBuilder[paramCount] = new MapSqlParameterSource(parameterMap);
            
            paramCount++;
            //When the batch size is reached do the update and reset the counter
            if (paramCount == this.batchSize) {
                resultsIndex = this.doBatchUpdate(sql, resultsIndex, allResults, paramCount, batchArgsBuilder, batchAverage);
                paramCount = 0;
            }
        }
        
        //Execute batch call for any remaining arguments
        if (paramCount > 0) {
            this.doBatchUpdate(sql, resultsIndex, allResults, paramCount, batchArgsBuilder, batchAverage);
        }
        
        if (logger.isDebugEnabled()) {
            final double avgUpdateTime = ((double) batchAverage.getTotal()) / ((double) parameters.size());
            logger.debug("Executed " + batchAverage.getCount() + " batches for " + parameters.size() + " updates in " + batchAverage.getTotal() + "ms, averaged " + AVG_FORMATTER.format(batchAverage.getAverage()) + "ms per batch and " + AVG_FORMATTER.format(avgUpdateTime) + "ms per update");
        }
        
        return allResults;
    }
    
    private int doBatchUpdate(String sql, int resultsIndex, final int[] allResults, int paramCount, final SqlParameterSource[] batchArgsBuilder, final Average updateAverage) {
        final long updateStart = System.currentTimeMillis();
        resultsIndex = this.doBatchUpdate(sql, resultsIndex, allResults, paramCount, batchArgsBuilder);
        final long updateTime = System.currentTimeMillis() - updateStart;
        
        updateAverage.add(updateTime);
        if (logger.isTraceEnabled()) {
            final double avgUpdateTime = ((double) updateTime) / ((double) batchArgsBuilder.length);
            logger.trace("Batched " + paramCount + " updates in " + updateTime + "ms, averaged " + AVG_FORMATTER.format(avgUpdateTime) + "ms per update");
        }
        
        return resultsIndex;
    }
    
    private int doBatchUpdate(String sql, int resultsIndex, int[] allResults, int parameterCount, SqlParameterSource[] parameters) {
        final SqlParameterSource[] batchArgs;
        if (parameterCount == parameters.length) {
            batchArgs = parameters;
        }
        else {
            batchArgs = new SqlParameterSource[parameterCount];
            System.arraycopy(parameters, 0, batchArgs, 0, parameterCount);
        }
        
        final int[] results = this.getSimpleJdbcTemplate().batchUpdate(sql, batchArgs);
        System.arraycopy(results, 0, allResults, resultsIndex, results.length);
        return resultsIndex + results.length;
    }
}