/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao.aggr;

import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class ConcurrentUsersDaoImpl extends BatchingJdbcDaoSupport implements ConcurrentUsersDao {
    private final String dateKeyParam;
    private final String timeKeyParam;
    private final String intervalKeyParam;
    private final String intervalStartDateParam;
    private final String timeSpanParam;
    private final String groupKeyParam;
    
    private final String userIdParam;
    private final String lastEventParam;
    
    //SQL to update the scratch table used to track last events
    private final String upsertConcurrentUserScratch;
    private final String deleteConcurrentUserScratch;
    
    //SQL to total unique logins and update the aggregate tables for by-group logins
    private final String upsertConcurrentUsers;
    
    private Interval longestInterval = Interval.DAY;
    
    public ConcurrentUsersDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/aggr/ConcurrentUsersDaoImplSql.xml", this.getClass().getClassLoader());
        this.dateKeyParam = sqlLoader.getSql("dateKeyParam");
        this.timeKeyParam = sqlLoader.getSql("timeKeyParam");
        this.intervalKeyParam = sqlLoader.getSql("intervalKeyParam");
        this.intervalStartDateParam = sqlLoader.getSql("intervalStartDateParam");
        this.timeSpanParam = sqlLoader.getSql("timeSpanParam");
        this.groupKeyParam = sqlLoader.getSql("groupKeyParam");
        
        this.userIdParam = sqlLoader.getSql("userIdParam");
        this.lastEventParam = sqlLoader.getSql("lastEventParam");
        
        this.upsertConcurrentUserScratch = sqlLoader.getSql("upsertConcurrentUserScratch");
        this.deleteConcurrentUserScratch = sqlLoader.getSql("deleteConcurrentUserScratch");
        
        this.upsertConcurrentUsers = sqlLoader.getSql("upsertConcurrentUsers");
    }
    
    public Interval getLongestInterval() {
        return longestInterval;
    }
    /**
     * The longest interval to generate an aggregate row for
     */
    public void setLongestInterval(Interval longestInterval) {
        this.longestInterval = longestInterval;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.aggr.ConcurrentUsersDao#saveLastEvents(java.util.Map)
     */
    @Transactional
    public void saveLastEvents(Map<Integer, Map<String, Date>> lastEvents) {
        if (lastEvents == null || lastEvents.size() == 0) {
            return;
        }
        
        final List<Map<String, Object>> batchParamters = new LinkedList<Map<String, Object>>();
        
        for (final Map.Entry<Integer, Map<String, Date>> lastEventEntry : lastEvents.entrySet()) {
            final Integer statsGroupKey = lastEventEntry.getKey();
            final Map<String, Date> lastGroupEvents = lastEventEntry.getValue();
        
            for (final Map.Entry<String, Date> lastEvent : lastGroupEvents.entrySet()) {
                final Date loginDate = lastEvent.getValue();
                final String userName = lastEvent.getKey();
                
                final Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(this.lastEventParam, loginDate);
                parameters.put(this.userIdParam, userName);
                parameters.put(this.groupKeyParam, statsGroupKey);
                
                batchParamters.add(parameters);
            }
        }
        
        if (batchParamters.size() > 0) {
            this.batchUpdate(this.upsertConcurrentUserScratch, batchParamters);
        }
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("saveLastEvents upserted " + batchParamters.size() + " rows.");
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.aggr.ConcurrentUsersDao#updateInterval(java.util.Map, java.util.Set)
     */
    @Transactional
    public void updateIntervals(Map<Interval, IntervalInfo> intervals, Set<Integer> groupKeys) {
        final List<Map<String, Object>> batchParamters = new LinkedList<Map<String, Object>>();
        
        for (final Map.Entry<Interval, IntervalInfo> intervalEntry : intervals.entrySet()) {
            final Interval interval = intervalEntry.getKey();
            final IntervalInfo intervalInfo = intervalEntry.getValue();
            
            //Grab the interval information
            final Date start = intervalInfo.getStart();
            final long intervalKey = this.getIntervalKey(interval);
            final int startDateKey = intervalInfo.getStartDateKey();
            final int startTimeKey = intervalInfo.getStartTimeKey();
            final int timeSpan = intervalInfo.getTimeSpan();
            
            for (final Integer statsGroupKey : groupKeys) {
                
                final Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(this.dateKeyParam, startDateKey);
                parameters.put(this.timeKeyParam, startTimeKey);
                parameters.put(this.intervalKeyParam, intervalKey);
                parameters.put(this.groupKeyParam, statsGroupKey);
                parameters.put(this.intervalStartDateParam, start);
                parameters.put(this.timeSpanParam, timeSpan);
                    
                batchParamters.add(parameters);
            }
        }
        
        if (batchParamters.size() > 0) {
            this.batchUpdate(this.upsertConcurrentUsers, batchParamters);
        }
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("updateIntervals upserted " + batchParamters.size() + " rows.");
        }
        
        if (this.longestInterval != null) {
            final IntervalInfo intervalInfo = intervals.get(this.longestInterval);
            if (intervalInfo != null) {
                final Date start = intervalInfo.getStart();
                final int rowsDeleted = this.getSimpleJdbcTemplate().update(this.deleteConcurrentUserScratch, start);
                
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug("Deleted " + rowsDeleted + " older than " + start + " for interval " + this.longestInterval + " from concurrent sessions table.");
                }
            }
        }
    }
}
