/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao.aggr;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class LoginAggregationsDaoImpl extends BatchingJdbcDaoSupport implements LoginAggregationsDao {
    //SQL Parameter Names
    private final String dateKeyParam;
    private final String timeKeyParam;
    private final String intervalKeyParam;
    private final String intervalStartDateParam;
    private final String timeSpanParam;
    private final String groupKeyParam;
    
    private final String userIdParam;
    private final String lastLoginParam;
    private final String additionalLoginsParam;
    
    //SQL to update the scratch table used to track unique logins
    private final String upsertPortalLoginScratch;
    private final String deletePortalLoginScratch;
    
    //SQL to total unique logins and update the aggregate tables for by-group logins
    private final String upsertLoginAggregate;
    
    private Interval longestInterval = Interval.YEAR;
    
    public LoginAggregationsDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/aggr/LoginAggregationsDaoImplSql.xml", this.getClass().getClassLoader());
        this.dateKeyParam = sqlLoader.getSql("dateKeyParam");
        this.timeKeyParam = sqlLoader.getSql("timeKeyParam");
        this.intervalKeyParam = sqlLoader.getSql("intervalKeyParam");
        this.intervalStartDateParam = sqlLoader.getSql("intervalStartDateParam");
        this.timeSpanParam = sqlLoader.getSql("timeSpanParam");
        this.groupKeyParam = sqlLoader.getSql("groupKeyParam");
        
        this.userIdParam = sqlLoader.getSql("userIdParam");
        this.lastLoginParam = sqlLoader.getSql("lastLoginParam");
        this.additionalLoginsParam = sqlLoader.getSql("additionalLoginsParam");

        this.upsertPortalLoginScratch = sqlLoader.getSql("upsertPortalLoginScratch");
        this.deletePortalLoginScratch = sqlLoader.getSql("deletePortalLoginScratch");
        
        this.upsertLoginAggregate = sqlLoader.getSql("upsertLoginAggregate");
    }
    
    public Interval getLongestInterval() {
        return longestInterval;
    }
    /**
     * The longest interval to generate an aggregate row for
     */
    public void setLongestInterval(Interval longestInterval) {
        this.longestInterval = longestInterval;
    }



    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.LoginAggregationsDao#saveLastGroupLogins(java.lang.Integer, java.util.Map)
     */
    @Transactional
    public void saveLastLogins(Map<Integer, Map<String, Date>> lastLogins) {
        if (lastLogins == null || lastLogins.size() == 0) {
            return;
        }
        
        final List<Map<String, Object>> batchParamters = new LinkedList<Map<String, Object>>();
        
        //Have to do updates one at a time since Oracle doesn't return per-parameter row affected counts
        for (final Map.Entry<Integer, Map<String, Date>> groupLastLoginsEntry : lastLogins.entrySet()) {
            final Integer statsGroupKey = groupLastLoginsEntry.getKey();
            final Map<String, Date> groupLastLogins = groupLastLoginsEntry.getValue();

            for (final Map.Entry<String, Date> lastLogin : groupLastLogins.entrySet()) {
                final Date loginDate = lastLogin.getValue();
                final String userName = lastLogin.getKey();
                
                final Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(this.lastLoginParam, loginDate);
                parameters.put(this.userIdParam, userName);
                parameters.put(this.groupKeyParam, statsGroupKey);
                
                batchParamters.add(parameters);
            }
        }

        this.batchUpdate(this.upsertPortalLoginScratch, batchParamters);
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("saveLastLogins upserted " + batchParamters.size() + " rows.");
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.LoginAggregationsDao#updateGroupInterval(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo, java.lang.Integer, int)
     */
    @Transactional
    public void updateIntervals(Map<Interval, IntervalInfo> intervals, Map<Integer, Integer> totalLogins) {
        final List<Map<String, Object>> batchParamters = new ArrayList<Map<String, Object>>(intervals.size() * totalLogins.size());
        
        for (final Map.Entry<Interval, IntervalInfo> intervalEntry : intervals.entrySet()) {
            final Interval interval = intervalEntry.getKey();
            final IntervalInfo intervalInfo = intervalEntry.getValue();
            
            //Grab the interval information
            final Date start = intervalInfo.getStart();
            final long intervalKey = this.getIntervalKey(interval);
            final int startDateKey = intervalInfo.getStartDateKey();
            final int startTimeKey = intervalInfo.getStartTimeKey();
            final int timeSpan = intervalInfo.getTimeSpan();
            
            for (final Map.Entry<Integer, Integer> totalLoginsEntry : totalLogins.entrySet()) {
                final Integer statsGroupKey = totalLoginsEntry.getKey();
                final Integer additionalLogins = totalLoginsEntry.getValue();
            
                final Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(this.dateKeyParam, startDateKey);
                parameters.put(this.timeKeyParam, startTimeKey);
                parameters.put(this.intervalKeyParam, intervalKey);
                parameters.put(this.timeSpanParam, timeSpan);
                parameters.put(this.groupKeyParam, statsGroupKey);
                parameters.put(this.intervalStartDateParam, start);
                parameters.put(this.additionalLoginsParam, additionalLogins);
                
                batchParamters.add(parameters);
            }
        }
        
        if (batchParamters.size() > 0) {
            this.batchUpdate(this.upsertLoginAggregate, batchParamters);
        }
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("updateIntervals upserted " + batchParamters.size() + " rows.");
        }
        
        if (this.longestInterval != null) {
            final IntervalInfo intervalInfo = intervals.get(this.longestInterval);
            if (intervalInfo != null) {
                final Date start = intervalInfo.getStart();
                final int rowsDeleted = this.getSimpleJdbcTemplate().update(this.deletePortalLoginScratch, start);
                
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug("Deleted " + rowsDeleted + " older than " + start + " for interval " + this.longestInterval + " from logins table.");
                }
            }
        }
    }
}
