/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.dao.aggr;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.jasig.portal.stats.om.LoginSummary;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class LoginFrequencyDaoImpl extends BatchingJdbcDaoSupport implements LoginFrequencyDao {
    private final ParameterizedRowMapper<FrequencyResult> mapper;
    
    private final String dateKeyParam;
    private final String timeKeyParam;
    private final String intervalKeyParam;
    private final String intervalStartDateParam;
    private final String timeSpanParam;
    private final String groupKeyParam;
    
    private final String userIdParam;
    private final String lastLoginParam;
    private final String groupKeysParam;
    private final String oneTimeParam;
    private final String twoTimesParam;
    private final String threeTimesParam;
    private final String sixTimesParam;
    private final String elevenTimesParam;
    private final String twentySixTimesParam;
    
    private final String insertPortalLoginFrequencyScratch;
    private final String deleteOldPortalLoginFrequencyScratch;

    private final String selectScratchLoginFrequencies;
    private final String upsertLoginFrequencies;
    
    
    private Interval longestInterval = Interval.DAY;
    
    
    public LoginFrequencyDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/dao/aggr/LoginFrequencyDaoImplSql.xml", this.getClass().getClassLoader());
        
        this.dateKeyParam = sqlLoader.getSql("dateKeyParam");
        this.timeKeyParam = sqlLoader.getSql("timeKeyParam");
        this.intervalKeyParam = sqlLoader.getSql("intervalKeyParam");
        this.intervalStartDateParam = sqlLoader.getSql("intervalStartDateParam");
        this.timeSpanParam = sqlLoader.getSql("timeSpanParam");
        this.groupKeyParam = sqlLoader.getSql("groupKeyParam");
        
        this.userIdParam = sqlLoader.getSql("userIdParam");
        this.lastLoginParam = sqlLoader.getSql("lastLoginParam");
        this.groupKeysParam = sqlLoader.getSql("groupKeysParam");
        this.oneTimeParam = sqlLoader.getSql("oneTimeParam");
        this.twoTimesParam = sqlLoader.getSql("twoTimesParam");
        this.threeTimesParam = sqlLoader.getSql("threeTimesParam");
        this.sixTimesParam = sqlLoader.getSql("sixTimesParam");
        this.elevenTimesParam = sqlLoader.getSql("elevenTimesParam");
        this.twentySixTimesParam = sqlLoader.getSql("twentySixTimesParam");
        
        this.insertPortalLoginFrequencyScratch = sqlLoader.getSql("insertPortalLoginFrequencyScratch");
        this.deleteOldPortalLoginFrequencyScratch = sqlLoader.getSql("deleteOldPortalLoginFrequencyScratch");

        this.selectScratchLoginFrequencies = sqlLoader.getSql("selectScratchLoginFrequencies");
        this.upsertLoginFrequencies = sqlLoader.getSql("upsertLoginFrequencies");
        
        this.mapper = new FrequencyResultRowMapper();
    }

    
    public Interval getLongestInterval() {
        return longestInterval;
    }
    /**
     * The longest interval to generate an aggregate row for
     */
    public void setLongestInterval(Interval longestInterval) {
        this.longestInterval = longestInterval;
    }


    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.aggr.LoginFrequencyDao#updateLoginFrequency(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo, java.util.List)
     */
    @Transactional
    public void saveLogins(List<LoginSummary> events) {
        final List<Map<String, Object>> batchInsertParamters = new LinkedList<Map<String, Object>>();
        
        //Build batch parameters map
        for (final LoginSummary summary : events) {
            final Map<String, Object> parameters = new HashMap<String, Object>();
            parameters.put(this.userIdParam, summary.userName);
            parameters.put(this.groupKeyParam, summary.groupKey);
            parameters.put(this.lastLoginParam, summary.activityDate);
            
            batchInsertParamters.add(parameters);
        }
        
        if (batchInsertParamters.size() > 0) {
            this.batchUpdate(this.insertPortalLoginFrequencyScratch, batchInsertParamters);
        }

        if (this.logger.isDebugEnabled()) {
            this.logger.debug("saveLogins inserted " + batchInsertParamters.size() + " into frequency table.");
        }
    }
    
    
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.dao.aggr.LoginFrequencyDao#updateLoginFrequencies(java.util.Map, java.util.Set)
     */
    @Transactional
    public void updateLoginFrequencies(Map<Interval, IntervalInfo> intervals, Set<Integer> groupKeys) {
        final SimpleJdbcTemplate simpleJdbcTemplate = this.getSimpleJdbcTemplate();
        
        final List<Map<String, Object>> batchParamters = new ArrayList<Map<String, Object>>(intervals.size() * groupKeys.size());
        
        for (final Map.Entry<Interval, IntervalInfo> intervalEntry : intervals.entrySet()) {
            final Interval interval = intervalEntry.getKey();
            final IntervalInfo intervalInfo = intervalEntry.getValue();
        
            final long intervalKey = this.getIntervalKey(interval);
            final int startDateKey = intervalInfo.getStartDateKey();
            final int startTimeKey = intervalInfo.getStartTimeKey();
            final Date start = intervalInfo.getStart();
            final int timeSpan = intervalInfo.getTimeSpan();
            
            //Calculate login frequencies from the scratch table
            final Map<String, Object> queryParameters = new HashMap<String, Object>();
            queryParameters.put(this.groupKeysParam, groupKeys);
            queryParameters.put(this.intervalStartDateParam, start);

            final List<FrequencyResult> frequencies = simpleJdbcTemplate.query(this.selectScratchLoginFrequencies, this.mapper, queryParameters);
            final Map<Integer, FrequencyCollator> collatedFrequencies = this.collateFrequencies(frequencies);
            
            for (final Integer statsGroupKey : groupKeys) {
                final FrequencyCollator frequencyCollator = collatedFrequencies.get(statsGroupKey);
                final long one = frequencyCollator.getOne();
                final long two = frequencyCollator.getTwo();
                final long threeToFive = frequencyCollator.getThreeToFive();
                final long sixToTen = frequencyCollator.getSixToTen();
                final long elevenToTwentyFive = frequencyCollator.getElevenToTwentyFive();
                final long twentySixOrMore = frequencyCollator.getTwentySixOrMore();
                
                final Map<String, Object> parameters = new HashMap<String, Object>();
                parameters.put(this.dateKeyParam, startDateKey);
                parameters.put(this.timeKeyParam, startTimeKey);
                parameters.put(this.intervalKeyParam, intervalKey);
                parameters.put(this.timeSpanParam, timeSpan);
                parameters.put(this.groupKeyParam, statsGroupKey);
                
                parameters.put(this.oneTimeParam, one);
                parameters.put(this.twoTimesParam, two);
                parameters.put(this.threeTimesParam, threeToFive);
                parameters.put(this.sixTimesParam, sixToTen);
                parameters.put(this.elevenTimesParam, elevenToTwentyFive);
                parameters.put(this.twentySixTimesParam, twentySixOrMore);
                
                batchParamters.add(parameters);
            }
        }
        
        if (batchParamters.size() > 0) {
            this.batchUpdate(this.upsertLoginFrequencies, batchParamters);
        }
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("updateLoginFrequencies upserted " + batchParamters.size() + " rows.");
        }
        
        if (this.longestInterval != null) {
            final IntervalInfo intervalInfo = intervals.get(this.longestInterval);
            if (intervalInfo != null) {
                final Date start = intervalInfo.getStart();
                final int rowsDeleted = this.getSimpleJdbcTemplate().update(this.deleteOldPortalLoginFrequencyScratch, start);
                
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug("Deleted " + rowsDeleted + " older than " + start + " for interval " + this.longestInterval + " from login frequency table.");
                }
            }
        }
    }
    
    protected Map<Integer, FrequencyCollator> collateFrequencies(List<FrequencyResult> frequencies) {
        final Map<Integer, FrequencyCollator> collatedFrequencies = new HashMap<Integer, FrequencyCollator>();
        
        for (final FrequencyResult frequency : frequencies) {
            FrequencyCollator frequencyCollator = collatedFrequencies.get(frequency.statsGroupKey);
            if (frequencyCollator == null) {
                frequencyCollator = new FrequencyCollator();
                collatedFrequencies.put(frequency.statsGroupKey, frequencyCollator);
            }
            
            frequencyCollator.add(frequency);
        }
        
        return collatedFrequencies;
    }
    
    //TODO this approach just feels ... messy
    private static class FrequencyCollator {
        private long one = 0;
        private long two = 0;
        private long threeToFive = 0;
        private long sixToTen = 0;
        private long elevenToTwentyFive = 0;
        private long twentySixOrMore = 0;
        
        public void add(FrequencyResult frequency) {
            if (frequency.loginCount == 1) {
                this.one += frequency.frequency;
            }
            else if (frequency.loginCount == 2) {
                this.two += frequency.frequency;
            }
            else if (frequency.loginCount >= 3 && frequency.loginCount <= 5) {
                this.threeToFive += frequency.frequency;
            }
            else if (frequency.loginCount >= 6 && frequency.loginCount <= 10) {
                this.sixToTen += frequency.frequency;
            }
            else if (frequency.loginCount >= 11 && frequency.loginCount <= 25) {
                this.elevenToTwentyFive += frequency.frequency;
            }
            else if (frequency.loginCount >= 26) {
                this.twentySixOrMore += frequency.frequency;
            }
        }
        
        public long getOne() {
            return one;
        }
        public long getTwo() {
            return two;
        }
        public long getThreeToFive() {
            return threeToFive;
        }
        public long getSixToTen() {
            return sixToTen;
        }
        public long getElevenToTwentyFive() {
            return elevenToTwentyFive;
        }
        public long getTwentySixOrMore() {
            return twentySixOrMore;
        }
    }

    private static class FrequencyResult {
        public final int statsGroupKey;
        public final long loginCount;
        public final long frequency;

        public FrequencyResult(int statsGroupKey, long loginCount, long frequency) {
            this.statsGroupKey = statsGroupKey;
            this.loginCount = loginCount;
            this.frequency = frequency;
        }
    }
    
    private static class FrequencyResultRowMapper implements ParameterizedRowMapper<FrequencyResult> {
        public FrequencyResult mapRow(ResultSet rs, int rowNum) throws SQLException {
            final int statsGroupKey = rs.getInt("PORTAL_GROUP_LOOKUP_KEY");
            final long loginCount = rs.getLong("LOGIN_COUNT");
            final long frequency = rs.getLong("FREQUENCY");
            
            return new FrequencyResult(statsGroupKey, loginCount, frequency);
        }
    }
}
