/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.read;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.ItemReader;
import org.springframework.batch.item.ItemStream;
import org.springframework.batch.item.ItemStreamException;
import org.springframework.batch.item.MarkFailedException;
import org.springframework.batch.item.NoWorkFoundException;
import org.springframework.batch.item.ParseException;
import org.springframework.batch.item.ResetFailedException;
import org.springframework.batch.item.UnexpectedInputException;

/**
 * Clone of {@link org.springframework.batch.item.support.AbstractBufferedItemReaderItemStream} that provides
 * better extension points
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public abstract class AbstractBufferedItemReaderItemStream implements ItemReader, ItemStream {
    protected final Log logger = LogFactory.getLog(this.getClass()); 
    
    private boolean shouldReadBuffer = false;
    private List<Object> itemBuffer = new ArrayList<Object>();
    private ListIterator<Object> itemBufferIterator = null;
    private int lastMarkedBufferIndex = 0;


    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemReader#mark()
     */
    public final void mark() throws MarkFailedException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("mark()");
        }

        if (!this.shouldReadBuffer) {
            this.itemBuffer.clear();
            this.itemBufferIterator = null;
            this.lastMarkedBufferIndex = 0;
        }
        else {
            this.lastMarkedBufferIndex = this.itemBufferIterator.nextIndex();
        }
    }

    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemReader#read()
     */
    public final Object read() throws Exception, UnexpectedInputException, NoWorkFoundException, ParseException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("read()");
        }
        
        if (this.shouldReadBuffer) {
            if (this.itemBufferIterator.hasNext()) {
                return this.itemBufferIterator.next();
            }

            // buffer is exhausted, continue reading from file
            this.shouldReadBuffer = false;
            this.itemBufferIterator = null;
        }

        final Object item = doRead();
        this.itemBuffer.add(item);

        return item;
    }

    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemReader#reset()
     */
    public final void reset() throws ResetFailedException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("reset()");
        }
        
        this.shouldReadBuffer = true;
        this.itemBufferIterator = this.itemBuffer.listIterator(this.lastMarkedBufferIndex);
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemStream#close(org.springframework.batch.item.ExecutionContext)
     */
    public final void close(ExecutionContext executionContext) throws ItemStreamException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("close()");
        }
        
        this.lastMarkedBufferIndex = 0;
        this.itemBufferIterator = null;
        this.shouldReadBuffer = false;
        this.itemBuffer.clear();

        try {
            doClose(executionContext);
        }
        catch (Exception e) {
            throw new ItemStreamException("Error while closing item reader", e);
        }
    }
    
    /**
     * Read next item from input.
     * @return item
     * @throws Exception
     */
    protected abstract Object doRead() throws Exception;

    /**
     * Close the resources opened in {@link #open(ExecutionContext)}.
     */
    protected abstract void doClose(ExecutionContext executionContext) throws Exception;

}
