/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.read;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;

import javax.sql.DataSource;

import org.apache.commons.lang.time.DateUtils;
import org.jasig.portal.stats.EventProcessingTracker;
import org.jasig.portal.stats.dao.JobDateStore;
import org.jasig.portal.stats.events.PortalEvent;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.ItemStreamException;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

/**
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class EventDateJdbcCursorItemReader extends ShutdownAwareJdbcCursorItemReader implements EventProcessingTracker {
    private String minDateSql;
    private int maxRange = -1;
    private int newEventBuffer = 10;

    private SimpleJdbcTemplate jdbcTemplate;
    private Date startDate;
    private Date lastEvent;


    public String getMinDateSql() {
        return this.minDateSql;
    }
    /**
     * The SQL used to find the first available event date for the configured event type
     */
    public void setMinDateSql(String minDateSql) {
        this.minDateSql = minDateSql;
    }

    public int getMaxRange() {
        return maxRange;
    }
    /**
     * The maximum number of minutes to include in the event date range. Defaults to 60.
     */
    public void setMaxRange(int maxRange) {
        this.maxRange = maxRange;
    }

    public int getNewEventBuffer() {
        return newEventBuffer;
    }
    /**
     * How close to current time should the event date range get in minutes. Defaults to 10. 
     */
    public void setNewEventBuffer(int newEventBuffer) {
        this.newEventBuffer = newEventBuffer;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.JdbcCursorItemReader#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        super.afterPropertiesSet();
        
        final DataSource dataSource = this.getDataSource();
        this.jdbcTemplate = new SimpleJdbcTemplate(dataSource);
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.JdbcCursorItemReader#prepareQuery(org.springframework.batch.item.ExecutionContext, java.sql.PreparedStatement)
     */
    @Override
    protected void setStatementParameters(ExecutionContext executionContext, PreparedStatement preparedStatement) throws SQLException {
        Date startEventDate = null;
        
        //See if a previous event date was stored in the ExecutionContext
        if (executionContext.containsKey(JobDateStore.STORED_DATE)) {
            startEventDate = (Date)executionContext.get(JobDateStore.STORED_DATE);
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("Loaded start date '" + startEventDate + "' from ExecutionContext");
            }
        }
        
        if (startEventDate == null) {
            //See if a last event value was stored in the database
            if (this.startDate != null) {
                startEventDate = this.startDate;
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug("Loaded start date '" + startEventDate + "' from JobParameters");
                }
            }
            else {
                //start from the beginning if no start date was stored
                startEventDate = this.getMinimumEventDate();
            }
        }

        final Date endEventDate;
        final Date rangeFromNow = DateUtils.addMinutes(new Date(), -this.newEventBuffer);
        
        if (this.maxRange <= 0) {
            endEventDate = rangeFromNow;
        }
        else {
            final Date rangeFromStart = DateUtils.addMinutes(startEventDate, this.maxRange);
    
            //Grab the smaller of the two values
            if (rangeFromStart.before(rangeFromNow)) {
                endEventDate = rangeFromStart;
            }
            else {
                endEventDate = rangeFromNow;
            }
        }
        
        if (this.logger.isInfoEnabled()) {
            this.logger.info("Setting query range from '" + startEventDate + "' to '" + endEventDate + "'");
        }
        
        preparedStatement.setInt(1, this.getMaxRows());
        preparedStatement.setTimestamp(2, new Timestamp(startEventDate.getTime()));
        preparedStatement.setTimestamp(3, new Timestamp(endEventDate.getTime()));
    }
    
    protected Date getMinimumEventDate() {
        if (this.minDateSql != null) {
            final Timestamp firstEventDate = this.jdbcTemplate.queryForObject(this.minDateSql, Timestamp.class);
            
            if (firstEventDate != null) {
                return DateUtils.addSeconds(new Date(firstEventDate.getTime()), -1);
            }
        }
        
        return new Date(0);
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.JdbcCursorItemReader#doRead()
     */
    @Override
    protected Object doRead() throws Exception {
        final Object item = super.doRead();
        if (item == null) {
            return null;
        }
        
        return item;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.EventProcessingTracker#setLastProcessedEvent(org.jasig.portal.stats.events.PortalEvent)
     */
    public void setLastProcessedEvent(PortalEvent event) {
        this.lastEvent = event.getActivityDate();
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.JdbcCursorItemReader#update(org.springframework.batch.item.ExecutionContext)
     */
    @Override
    public void update(ExecutionContext executionContext) throws ItemStreamException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("update()");
        }
        
        executionContext.put(JobDateStore.STORED_DATE, this.lastEvent);
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Stored last processed date '" + this.lastEvent + "' in ExecutionContext");
        }
    }

    /* (non-Javadoc)
     * @see org.springframework.batch.core.StepExecutionListener#beforeStep(org.springframework.batch.core.StepExecution)
     */
    @Override
    public void beforeStep(StepExecution stepExecution) {
        super.beforeStep(stepExecution);
        
        final JobParameters jobParameters = stepExecution.getJobParameters();
        this.startDate = jobParameters.getDate(JobDateStore.STORED_DATE);
    }
}
