/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.read;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;

import javax.sql.DataSource;

import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.ItemStreamException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.dao.InvalidDataAccessResourceUsageException;
import org.springframework.jdbc.SQLWarningException;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.support.JdbcUtils;
import org.springframework.jdbc.support.SQLErrorCodeSQLExceptionTranslator;
import org.springframework.jdbc.support.SQLExceptionTranslator;
import org.springframework.jdbc.support.SQLStateSQLExceptionTranslator;
import org.springframework.util.Assert;

/**
 * Clone of {@link org.springframework.batch.item.database.JdbcCursorItemReader} that provides extension points
 * for dynamically creating the SQL or parameters used for the driving query. 
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class JdbcCursorItemReader extends AbstractBufferedItemReaderItemStream implements InitializingBean {
    public static final int VALUE_NOT_SET = -1;
    
    private DataSource dataSource;
    private RowMapper mapper;
    private String sql;
    private int fetchSize = VALUE_NOT_SET;
    private int maxRows = VALUE_NOT_SET;
    private int queryTimeout = VALUE_NOT_SET;
    private boolean ignoreWarnings = true;
    private boolean verifyCursorPosition = true;
    
    private boolean initialized = false;
    private SQLExceptionTranslator exceptionTranslator;
    private Connection connection;
    private PreparedStatement preparedStatement;
    private ResultSet rs;
    private int currentRow;
    

    public DataSource getDataSource() {
        return dataSource;
    }
    /**
     * Public setter for the data source for injection purposes.
     * 
     * @param dataSource
     */
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public RowMapper getMapper() {
        return mapper;
    }
    /**
     * Set the RowMapper to be used for all calls to read().
     * 
     * @param mapper
     */
    public void setMapper(RowMapper mapper) {
        this.mapper = mapper;
    }

    public String getSql() {
        return sql;
    }
    /**
     * Set the SQL statement to be used when creating the cursor. This statement
     * should be a complete and valid SQL statement, as it will be run directly
     * without any modification.
     * 
     * @param sql
     */
    public void setSql(String sql) {
        this.sql = sql;
    }

    public int getFetchSize() {
        return fetchSize;
    }
    /**
     * Gives the JDBC driver a hint as to the number of rows that should be
     * fetched from the database when more rows are needed for this
     * <code>ResultSet</code> object. If the fetch size specified is zero, the
     * JDBC driver ignores the value.
     * 
     * @param fetchSize the number of rows to fetch
     * @see ResultSet#setFetchSize(int)
     */
    public void setFetchSize(int fetchSize) {
        this.fetchSize = fetchSize;
    }

    public int getMaxRows() {
        return maxRows;
    }
    /**
     * Sets the limit for the maximum number of rows that any
     * <code>ResultSet</code> object can contain to the given number.
     * 
     * @param maxRows the new max rows limit; zero means there is no limit
     * @see Statement#setMaxRows(int)
     */
    public void setMaxRows(int maxRows) {
        this.maxRows = maxRows;
    }

    public int getQueryTimeout() {
        return queryTimeout;
    }
    /**
     * Sets the number of seconds the driver will wait for a
     * <code>Statement</code> object to execute to the given number of
     * seconds. If the limit is exceeded, an <code>SQLException</code> is
     * thrown.
     * 
     * @param queryTimeout seconds the new query timeout limit in seconds; zero
     * means there is no limit
     * @see Statement#setQueryTimeout(int)
     */
    public void setQueryTimeout(int queryTimeout) {
        this.queryTimeout = queryTimeout;
    }

    public boolean isIgnoreWarnings() {
        return ignoreWarnings;
    }
    /**
     * Set whether SQLWarnings should be ignored (only logged) or exception
     * should be thrown.
     * 
     * @param ignoreWarnings if TRUE, warnings are ignored
     */
    public void setIgnoreWarnings(boolean ignoreWarnings) {
        this.ignoreWarnings = ignoreWarnings;
    }

    public boolean isVerifyCursorPosition() {
        return verifyCursorPosition;
    }
    /**
     * Allow verification of cursor position after current row is processed by
     * RowMapper or RowCallbackHandler. Default value is TRUE.
     * 
     * @param verifyCursorPosition if true, cursor position is verified
     */
    public void setVerifyCursorPosition(boolean verifyCursorPosition) {
        this.verifyCursorPosition = verifyCursorPosition;
    }

    /* (non-Javadoc)
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        Assert.notNull(this.dataSource, "DataSOurce must be provided");
        Assert.notNull(this.sql, "Sql must be provided");
        Assert.notNull(this.mapper, "RowMapper must be provided");
    }

    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemStream#open(org.springframework.batch.item.ExecutionContext)
     */
    public final void open(ExecutionContext executionContext) throws ItemStreamException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("open()");
        }
        
        Assert.state(!this.initialized, "Stream is already initialized.  Close before re-opening.");
        Assert.isNull(this.rs, "ResultSet still open!  Close before re-opening.");
        
        Assert.state(this.dataSource != null, "DataSource must not be null.");

        try {
            this.connection = this.dataSource.getConnection();
            this.preparedStatement = this.connection.prepareStatement(this.sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY, ResultSet.HOLD_CURSORS_OVER_COMMIT);
            
            this.applyStatementSettings(this.preparedStatement);
            
            this.setStatementParameters(executionContext, preparedStatement);
            
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("Starting query execution");
            }
            final long queryStart = System.currentTimeMillis();
            
            this.rs = this.preparedStatement.executeQuery();
            
            final long queryLength = System.currentTimeMillis() - queryStart;
            if (this.logger.isInfoEnabled()) {
                this.logger.info("Query execution complete in " + queryLength + "ms");
            }
            
            handleWarnings(this.preparedStatement.getWarnings());
        }
        catch (SQLException se) {
            close(null);
            throw getExceptionTranslator().translate("Executing query", sql, se);
        }
        
        this.currentRow = 0;
        this.initialized = true;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.AbstractBufferedItemReaderItemStream#doRead()
     */
    @Override
    protected Object doRead() throws Exception {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("doRead()");
        }
        
        try {
            if (!rs.next()) {
                return null;
            }
            
            this.currentRow++;
            final Object item = this.mapper.mapRow(this.rs, this.currentRow);
            this.verifyCursorPosition(this.currentRow);

            return item;
        }
        catch (SQLException se) {
            throw getExceptionTranslator().translate("Attempt to process next row failed", sql, se);
        }
    }

    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemStream#update(org.springframework.batch.item.ExecutionContext)
     */
    public void update(ExecutionContext executionContext) throws ItemStreamException {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("update()");
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.AbstractBufferedItemReaderItemStream#doClose(org.springframework.batch.item.ExecutionContext)
     */
    @Override
    protected void doClose(ExecutionContext executionContext) throws Exception {
        if (this.logger.isTraceEnabled()) {
            this.logger.trace("doClose()");
        }
        
        this.initialized = false;
        JdbcUtils.closeResultSet(this.rs);
        JdbcUtils.closeStatement(this.preparedStatement);
        JdbcUtils.closeConnection(this.connection);
        this.rs = null;
        this.preparedStatement = null;
        this.connection = null;
        this.currentRow = 0;
    }
    
    
    /**
     * Prepare the given JDBC Statement (or PreparedStatement or
     * CallableStatement), applying statement settings such as fetch size, max
     * rows, and query timeout. @param stmt the JDBC Statement to prepare
     * @throws SQLException
     * 
     * @see #setFetchSize
     * @see #setMaxRows
     * @see #setQueryTimeout
     */
    protected void applyStatementSettings(PreparedStatement stmt) throws SQLException {
        if (fetchSize != VALUE_NOT_SET) {
            stmt.setFetchSize(fetchSize);
            stmt.setFetchDirection(ResultSet.FETCH_FORWARD);
        }
        if (maxRows != VALUE_NOT_SET) {
            stmt.setMaxRows(maxRows);
        }
        if (queryTimeout != VALUE_NOT_SET) {
            stmt.setQueryTimeout(queryTimeout);
        }
    }
    
    /**
     * Throw a SQLWarningException if we're not ignoring warnings, else log the
     * warnings (at debug level).
     * 
     * @param warning the warnings object from the current statement. May be
     * <code>null</code>, in which case this method does nothing.
     * 
     * @see org.springframework.jdbc.SQLWarningException
     */
    protected void handleWarnings(SQLWarning warnings) throws SQLWarningException {
        if (this.ignoreWarnings) {
            SQLWarning warningToLog = warnings;
            while (warningToLog != null) {
                this.logger.debug("SQLWarning ignored: SQL state '" + warningToLog.getSQLState() + "', error code '"
                        + warningToLog.getErrorCode() + "', message [" + warningToLog.getMessage() + "]");
                warningToLog = warningToLog.getNextWarning();
            }
        }
        else if (warnings != null) {
            throw new SQLWarningException("Warning not ignored", warnings);
        }
    }
    
    /**
     * Return the exception translator for this instance.
     * 
     * Creates a default SQLErrorCodeSQLExceptionTranslator for the specified
     * DataSource if none is set.
     */
    protected SQLExceptionTranslator getExceptionTranslator() {
        if (this.exceptionTranslator == null) {
            if (this.dataSource != null) {
                this.exceptionTranslator = new SQLErrorCodeSQLExceptionTranslator(this.dataSource);
            }
            else {
                this.exceptionTranslator = new SQLStateSQLExceptionTranslator();
            }
        }
        return this.exceptionTranslator;
    }

    /**
     * Check the result set is in synch with the currentRow attribute. This is
     * important to ensure that the user hasn't modified the current row.
     */
    protected void verifyCursorPosition(long expectedCurrentRow) throws SQLException {
        if (verifyCursorPosition) {
            if (expectedCurrentRow != this.rs.getRow()) {
                throw new InvalidDataAccessResourceUsageException("Unexpected cursor position change.");
            }
        }
    }
    
    @SuppressWarnings("unused")
    protected void setStatementParameters(ExecutionContext executionContext, PreparedStatement preparedStatement) throws SQLException {
        //noop
    }
}
