/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.EventProcessingTracker;
import org.jasig.portal.stats.IntervalHelper;
import org.jasig.portal.stats.events.PortalEvent;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;
import org.springframework.batch.item.ItemWriter;

/**
 * Base portal event writer, casts generic items to PortalEvent then does the following:
 * - Tracks time intervals and sends notifications when interval boundries are crossed
 * - Filters events based on calls to {@link #supports(org.jasig.portal.stats.events.EventType)}
 * - Filters interval notifications based on calls to {@link #handles(Interval)}
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public abstract class BaseEventWriter<E extends PortalEvent> implements ItemWriter, IntervalAwareItemWriter<E> {
    private final Map<Interval, IntervalInfo> intervalInfo = new HashMap<Interval, IntervalInfo>();
    private E buffer = null;
    
    private EventProcessingTracker eventProcessingTracker;
    private IntervalHelper intervalHelper;
    private Set<Interval> intervals = new LinkedHashSet<Interval>(Arrays.asList(Interval.values()));
    
    
    public EventProcessingTracker getEventProcessingTracker() {
        return eventProcessingTracker;
    }
    /**
     * Tracker to notify after each event is successfully processed
     */
    public void setEventProcessingTracker(EventProcessingTracker eventProcessingTracker) {
        this.eventProcessingTracker = eventProcessingTracker;
    }
    
    
    public IntervalHelper getIntervalHelper() {
        return intervalHelper;
    }
    /**
     * Helper class used to determine the correct interval information
     */
    public void setIntervalHelper(IntervalHelper intervalHelper) {
        this.intervalHelper = intervalHelper;
    }

    public Set<Interval> getIntervals() {
        return intervals;
    }
    /**
     * Intervals that should be monitored and trigger calls to {@link #handleIntervalBoundry(Interval, IntervalInfo)}
     */
    public void setIntervals(Set<Interval> intervals) {
        this.intervals = intervals;
    }

    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemWriter#clear()
     */
    public final void clear() throws ClearFailedException {
        this.buffer = null;
        this.intervalInfo.clear();
        this.doClear();
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemWriter#flush()
     */
    public final void flush() throws FlushFailedException {
        this.doFlush(this.intervalInfo);
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.item.ItemWriter#write(java.lang.Object)
     */
    @SuppressWarnings("unchecked")
    public final void write(Object item) throws Exception {
        final E event = this.buffer;
        this.buffer = (E)item;
        
        if (event != null) {
            //If events are equal it is due to 'old' data in the buffer
            if (event.equals(this.buffer) || !this.supports(event.getType())) {
                return;
            }
            this.doWrite(event);
        }
        

        final Date activityDate = this.buffer.getActivityDate();
        for (final Interval interval : intervals) {
            IntervalInfo intervalDates = this.intervalInfo.get(interval);
            
            if (intervalDates == null) {
                intervalDates = this.intervalHelper.getIntervalDates(interval, activityDate);
                this.intervalInfo.put(interval, intervalDates);
            }
            else if (!intervalDates.getEnd().after(activityDate)) {
                this.handleIntervalBoundry(interval, this.intervalInfo);
                
                intervalDates = this.intervalHelper.getIntervalDates(interval, activityDate); 
                this.intervalInfo.put(interval, intervalDates);
            }
        }
        
        if (event != null && this.eventProcessingTracker != null) {
            this.eventProcessingTracker.setLastProcessedEvent(event);
        }
    }
}
