/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.PortalEvent;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Delegates all methods from {@link IntervalAwareItemWriter} to each specified delegate in the order configured
 * in the list.
 * 
 * No exception handling is done by this class
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class CompositeEventWriter<T extends PortalEvent> extends BaseEventWriter<T> {
    private List<IntervalAwareItemWriter<T>> delegates;
    
    public List<IntervalAwareItemWriter<T>> getDelegates() {
        return delegates;
    }
    /**
     * The {@link IntervalAwareItemWriter}s to delegat to in their configured order.
     */
    public void setDelegates(List<IntervalAwareItemWriter<T>> delegates) {
        this.delegates = delegates;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.BaseEventWriter#doWrite(org.jasig.portal.stats.events.PortalEvent)
     */
    public void doWrite(T event) throws Exception {
        for (final IntervalAwareItemWriter<T> itemWriter : this.delegates) {
            if (itemWriter.supports(event.getType())) {
                itemWriter.doWrite(event);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.BaseEventWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalHelper.IntervalDates)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        for (final IntervalAwareItemWriter<T> itemWriter : this.delegates) {
            if (itemWriter.handles(interval)) {
                final Map<Interval, IntervalInfo> filteredIntervals = this.filterIntervals(itemWriter, intervals);
                itemWriter.handleIntervalBoundry(interval, filteredIntervals);
            }
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        for (final IntervalAwareItemWriter<T> itemWriter : this.delegates) {
            itemWriter.doClear();
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervalInfo) throws FlushFailedException {
        for (final IntervalAwareItemWriter<T> itemWriter : this.delegates) {
            final Map<Interval, IntervalInfo> filteredIntervals = this.filterIntervals(itemWriter, intervalInfo);
            itemWriter.doFlush(filteredIntervals);
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#supports(org.jasig.portal.stats.events.EventType)
     */
    public boolean supports(EventType eventType) {
        return true;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handles(org.jasig.portal.stats.Interval)
     */
    public boolean handles(Interval interval) {
        return true;
    }
    
    protected Map<Interval, IntervalInfo> filterIntervals(IntervalAwareItemWriter<T> itemWriter, Map<Interval, IntervalInfo> intervals) {
        final Map<Interval, IntervalInfo> filteredIntervals = new HashMap<Interval, IntervalInfo>();
        
        for (final Map.Entry<Interval, IntervalInfo> intervalEntry : intervals.entrySet()) {
            final Interval interval = intervalEntry.getKey();
            
            if (itemWriter.handles(interval)) {
                filteredIntervals.put(interval, intervalEntry.getValue());
            }
        }
        
        return filteredIntervals;
    }
}
