/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.jasig.portal.stats.dao.aggr.ConcurrentUsersDao;
import org.jasig.portal.stats.events.PortalEvent;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Tracks and summarizes SessionCreatedEvents for login information. Summaries are done in memory and then flushed
 * on each interval boundry
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class ConcurrentUserAggregateWriter extends BaseIntervalAwareItemWriter<PortalEvent> {
    private boolean dirty = false;
    private final Map<Integer, Map<String, Date>> lastEvents = new HashMap<Integer, Map<String, Date>>();
    
    private ConcurrentUsersDao concurrentUsersDao;
    
    
    public ConcurrentUsersDao getConcurrentUsersDao() {
        return concurrentUsersDao;
    }
    /**
     * @param concurrentUsersDao the concurrentUsersDao to set
     */
    public void setConcurrentUsersDao(ConcurrentUsersDao concurrentUsersDao) {
        this.concurrentUsersDao = concurrentUsersDao;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        this.dirty = false;
        this.lastEvents.clear();
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException {
        if (this.dirty) {
            this.concurrentUsersDao.saveLastEvents(this.lastEvents);
            this.concurrentUsersDao.updateIntervals(intervals, this.lastEvents.keySet());
            
            //clear in-memory data tracking
            this.doClear();
        }
    }

    /*
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doWrite(java.lang.Object)
     */
    public void doWrite(PortalEvent portalEvent) throws Exception {
        this.dirty = true;

        final String userName = portalEvent.getUserName();
        final Date loginDate = portalEvent.getActivityDate();

        //Update per-group count & unique Map
        final Map<String, Integer> groups = portalEvent.getGroups();
        if (groups != null) {
            for (final Integer statsGroupKey : groups.values()) {
                Map<String, Date> lastUserGroupLogins = this.lastEvents.get(statsGroupKey);
                if (lastUserGroupLogins == null) {
                    lastUserGroupLogins = new HashMap<String, Date>();
                    this.lastEvents.put(statsGroupKey, lastUserGroupLogins);
                }
                this.putIfNewer(lastUserGroupLogins, userName, loginDate);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        this.doFlush(intervals);
    }
    
    protected void putIfNewer(Map<String, Date> map, String key, Date value) {
        final Date previousLoginDate = map.get(key);
        if (previousLoginDate == null || value.after(previousLoginDate)) {
            map.put(key, value);
        }
    }
}
