/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.Map;

import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.PortalEvent;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Base interface for portal event writers. Has methods to handle flushing on interval boundries and filtering events
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 * @param <T>
 */
public interface IntervalAwareItemWriter<T extends PortalEvent> {
    /**
     * Flush any buffers that are being held. This will usually be performed
     * prior to committing any transactions.
     * @throws FlushFailedException in case of an error.  If this exception is
     * thrown the writer may be in an inconsistent state and manual intervention
     * might be required to reconcile the data with persistent output.
     * 
     * @param intervals The intervals that have been seen since the last call to clear()
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException;

    /**
     * Clear any buffers that are being held. This will usually be performed
     * prior to rolling back any transactions.
     * @throws ClearFailedException in case of an error. If this exception is
     * thrown the writer may be in an inconsistent state and manual intervention
     * might be required to reconcile the data with persistent output.
     */
    public void doClear() throws ClearFailedException;
    
    /**
     * Process the supplied data element. Will be called multiple times during a
     * larger batch operation. Will not be called with null data in normal
     * operation.
     * 
     * @throws Exception if there are errors. If the writer is used inside a
     * retry or a batch the framework will catch the exception and convert or
     * rethrow it as appropriate.
     */
    public void doWrite(T item) throws Exception;

    /**
     * If the next event crosses a time interval boundary, the boundary crossed along with its start and end
     * dates are passed. Called before {@link #doWrite(PortalEvent)} is called on the event that crosses
     * the boundary
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals);
    
    /**
     * @return true if this writer supports the specified event type
     */
    public boolean supports(EventType eventType);
    
    /**
     * @return true if this writer handles the specified interval
     */
    public boolean handles(Interval interval);
}