/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.Date;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.dao.aggr.LoginFrequencyDao;
import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.SessionCreatedEvent;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.jasig.portal.stats.om.LoginSummary;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Tracks and summarizes SessionCreatedEvents for login information. Summaries are done in memory and then flushed
 * on each interval boundry
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class LoginFrequencyAggregateWriter extends BaseIntervalAwareItemWriter<SessionCreatedEvent> {
    private boolean dirty = false;
    private List<LoginSummary> events = new LinkedList<LoginSummary>();
    private Set<Integer> groupKeys = new HashSet<Integer>();
    
    private LoginFrequencyDao loginFrequencyDao;
    
    
    
    public LoginFrequencyAggregateWriter() {
        final Set<EventType> supportedItems = EnumSet.of(EventType.SESSION_CREATED);
        this.setSupportedEvents(supportedItems);
    }

    public LoginFrequencyDao getLoginFrequencyDao() {
        return loginFrequencyDao;
    }
    /**
     * @param loginFrequencyDao the loginFrequencyDao to set
     */
    public void setLoginFrequencyDao(LoginFrequencyDao loginFrequencyDao) {
        this.loginFrequencyDao = loginFrequencyDao;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        this.dirty = false;
        this.events.clear();
        this.groupKeys.clear();
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException {
        if (this.dirty) {
            this.loginFrequencyDao.saveLogins(this.events);
            this.loginFrequencyDao.updateLoginFrequencies(intervals, this.groupKeys);

            //clear in-memory data tracking
            this.doClear();
        }
    }

    /*
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doWrite(java.lang.Object)
     */
    public void doWrite(SessionCreatedEvent sessionCreatedEvent) throws Exception {
        this.dirty = true;
        
        final String userName = sessionCreatedEvent.getUserName();
        final Date activityDate = sessionCreatedEvent.getActivityDate();
        final Map<String, Integer> groups = sessionCreatedEvent.getGroups();
        
        for (final Integer groupKey : groups.values()) {
            this.events.add(new LoginSummary(userName, activityDate, groupKey));
            this.groupKeys.add(groupKey);
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        this.doFlush(intervals);
    }
}
