/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.Date;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.dao.aggr.LoginAggregationsDao;
import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.SessionCreatedEvent;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Tracks and summarizes SessionCreatedEvents for login information. Summaries are done in memory and then flushed
 * on each interval boundry
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class LoginsAggregateWriter extends BaseIntervalAwareItemWriter<SessionCreatedEvent> {
    private boolean dirty = false;
    private final Map<Integer, Integer> totalLogins = new HashMap<Integer, Integer>();
    private final Map<Integer, Map<String, Date>> lastLogins = new HashMap<Integer, Map<String, Date>>();
    
    private LoginAggregationsDao loginAggregationsDao;
    
    
    public LoginsAggregateWriter() {
        final Set<EventType> supportedItems = EnumSet.of(EventType.SESSION_CREATED);
        this.setSupportedEvents(supportedItems);
    }
    
    public LoginAggregationsDao getLoginAggregationsDao() {
        return loginAggregationsDao;
    }
    /**
     * @param loginAggregationsDao DAO to use to store login aggregations
     */
    public void setLoginAggregationsDao(LoginAggregationsDao loginAggregationsDao) {
        this.loginAggregationsDao = loginAggregationsDao;
    }
    

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        this.dirty = false;
        this.totalLogins.clear();
        this.lastLogins.clear();
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException {
        if (this.dirty) {
            this.loginAggregationsDao.saveLastLogins(this.lastLogins);
            this.loginAggregationsDao.updateIntervals(intervals, this.totalLogins);
            
            //clear in-memory data tracking
            this.doClear();
        }
    }

    /*
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doWrite(java.lang.Object)
     */
    public void doWrite(SessionCreatedEvent sessionCreatedEvent) throws Exception {
        this.dirty = true;

        final String userName = sessionCreatedEvent.getUserName();
        final Date loginDate = sessionCreatedEvent.getActivityDate();

        //Update per-group count & unique Map
        final Map<String, Integer> groups = sessionCreatedEvent.getGroups();
        if (groups != null) {
            for (final Integer statsGroupKey : groups.values()) {
                Map<String, Date> lastUserGroupLogins = this.lastLogins.get(statsGroupKey);
                if (lastUserGroupLogins == null) {
                    lastUserGroupLogins = new HashMap<String, Date>();
                    this.lastLogins.put(statsGroupKey, lastUserGroupLogins);
                }
                this.putIfNewer(lastUserGroupLogins, userName, loginDate);

                final Integer groupLogins = this.totalLogins.get(statsGroupKey);
                this.totalLogins.put(statsGroupKey, groupLogins != null ? groupLogins + 1 : 1);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        this.doFlush(intervals);
    }
    
    protected void putIfNewer(Map<String, Date> map, String key, Date value) {
        final Date previousLoginDate = map.get(key);
        if (previousLoginDate == null || value.after(previousLoginDate)) {
            map.put(key, value);
        }
    }
}
