/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.dao.aggr.PortletAggregateDao;
import org.jasig.portal.stats.events.ChannelEvent;
import org.jasig.portal.stats.events.ChannelRenderedEvent;
import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.PortletActionEvent;
import org.jasig.portal.stats.om.ChannelData;
import org.jasig.portal.stats.om.ChannelInfo;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Tracks and summarizes channel rendered, targetd and action events for channel information. Summaries are done in
 * memory and then flushed on each interval boundry
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class PortletAggregateWriter extends BaseIntervalAwareItemWriter<ChannelEvent> {
    private boolean dirty = false;
    private final Map<Integer, Map<Long, ChannelData>> channelDataMap = new HashMap<Integer, Map<Long,ChannelData>>();
    
    private PortletAggregateDao portletAggregateDao;
    
    public PortletAggregateWriter() {
        final Set<EventType> supportedItems = EnumSet.of(EventType.CHANNEL_RENDERED, EventType.CHANNEL_TARGETED, EventType.PORTLET_ACTION);
        this.setSupportedEvents(supportedItems);
    }
    
    public PortletAggregateDao getPortletAggregateDao() {
        return portletAggregateDao;
    }
    /**
     * @param portletAggregateDao the portletAggregateDao to set
     */
    public void setPortletAggregateDao(PortletAggregateDao portletAggregateDao) {
        this.portletAggregateDao = portletAggregateDao;
    }


    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        this.channelDataMap.clear();
        this.dirty = false;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException {
        if (this.dirty) {
            this.portletAggregateDao.updateIntervals(intervals, this.channelDataMap);
            
            //clear in-memory data tracking
            this.doClear();
        }
    }
    
    /*
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doWrite(java.lang.Object)
     */
    public void doWrite(ChannelEvent channelEvent) throws Exception {
        this.dirty = true;

        final ChannelInfo channelInfo = channelEvent.getChannelInfo();
        final Long channelStatsKey = channelInfo.getChannelStatsKey();
        
        final Map<String, Integer> groups = channelEvent.getGroups();
        for (final Integer statsGroupKey : groups.values()) {
            
            Map<Long, ChannelData> groupChannelDataMap = this.channelDataMap.get(statsGroupKey);
            if (groupChannelDataMap == null) {
                groupChannelDataMap = new HashMap<Long, ChannelData>();
                this.channelDataMap.put(statsGroupKey, groupChannelDataMap);
            }
            
            ChannelData channelData = groupChannelDataMap.get(channelStatsKey);
            if (channelData == null) {
                channelData = new ChannelData();
                groupChannelDataMap.put(channelStatsKey, channelData);
            }
            
            switch (channelEvent.getType()) {
                case CHANNEL_RENDERED: {
                    final ChannelRenderedEvent channelRenderedEvent = (ChannelRenderedEvent)channelEvent;
                    
                    if (channelRenderedEvent.isCached()) {
                        channelData.cachedCount++;
                    }
                    else {
                        channelData.renderTimes.addTime(channelRenderedEvent.getTime());
                    }
                } break;
                
                case CHANNEL_TARGETED: {
                    channelData.targetedCount++;
                } break;
                
                case PORTLET_ACTION: {
                    final PortletActionEvent portletActionEvent = (PortletActionEvent)channelEvent;
                    channelData.actionTimes.addTime(portletActionEvent.getTime());
                } break;
            }
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        this.doFlush(intervals);
    }
}
