/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.item.write;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.stats.dao.aggr.TabAggregateDao;
import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.PageRenderedEvent;
import org.jasig.portal.stats.om.CountAndTime;
import org.jasig.portal.stats.om.Interval;
import org.jasig.portal.stats.om.IntervalInfo;
import org.jasig.portal.stats.om.TabInfo;
import org.springframework.batch.item.ClearFailedException;
import org.springframework.batch.item.FlushFailedException;

/**
 * Tracks and summarizes PageRenderEvents for tab information. Summaries are done in memory and then flushed
 * on each interval boundry
 * 
 * This class is NOT thread safe
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class TabAggregateWriter extends BaseIntervalAwareItemWriter<PageRenderedEvent> {
    private boolean dirty = false;
    //group id to channel id to channel data
    private final Map<Integer, Map<Long, CountAndTime>> tabDataMap = new HashMap<Integer, Map<Long, CountAndTime>>();
    
    private TabAggregateDao tabAggregateDao;
    
    public TabAggregateWriter() {
        final Set<EventType> supportedItems = EnumSet.of(EventType.PAGE_RENDER_TIME);
        this.setSupportedEvents(supportedItems);
    }
    
    public TabAggregateDao getTabAggregateDao() {
        return this.tabAggregateDao;
    }
    /**
     * @param tabAggregateDao the tabAggregateDao to set
     */
    public void setTabAggregateDao(TabAggregateDao tabAggregateDao) {
        this.tabAggregateDao = tabAggregateDao;
    }


    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doClear()
     */
    public void doClear() throws ClearFailedException {
        this.tabDataMap.clear();
        this.dirty = false;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doFlush(java.util.Map)
     */
    public void doFlush(Map<Interval, IntervalInfo> intervals) throws FlushFailedException {
        if (this.dirty) {
            this.tabAggregateDao.updateIntervals(intervals, this.tabDataMap);
            
            //clear in-memory data tracking
            this.doClear();
        }
    }
    
    /*
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#doWrite(java.lang.Object)
     */
    public void doWrite(PageRenderedEvent pageRenderedEvent) throws Exception {
        this.dirty = true;

        final TabInfo tabInfo = pageRenderedEvent.getTabInfo();
        final long tabStatsKey = tabInfo.getTabStatsKey();
        
        final Map<String, Integer> groups = pageRenderedEvent.getGroups();
        for (final Integer statsGroupKey : groups.values()) {
            Map<Long, CountAndTime> groupTabDataMap = this.tabDataMap.get(statsGroupKey);
            if (groupTabDataMap == null) {
                groupTabDataMap = new HashMap<Long, CountAndTime>();
                this.tabDataMap.put(statsGroupKey, groupTabDataMap);
            }
            
            CountAndTime tabData = groupTabDataMap.get(tabStatsKey);
            if (tabData == null) {
                tabData = new CountAndTime();
                groupTabDataMap.put(tabStatsKey, tabData);
            }
            
            long pageRenderTime = pageRenderedEvent.getTime();
            tabData.addTime(pageRenderTime);
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.item.write.IntervalAwareItemWriter#handleIntervalBoundry(org.jasig.portal.stats.Interval, org.jasig.portal.stats.IntervalInfo)
     */
    public void handleIntervalBoundry(Interval interval, Map<Interval, IntervalInfo> intervals) {
        this.doFlush(intervals);
    }
}
