/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.mappers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.Map;

import org.jasig.portal.stats.dao.ChannelInfoLoader;
import org.jasig.portal.stats.dao.TabInfoLoader;
import org.jasig.portal.stats.events.ChannelEvent;
import org.jasig.portal.stats.events.ChannelRenderedEvent;
import org.jasig.portal.stats.events.ChannelTargetedEvent;
import org.jasig.portal.stats.events.EventType;
import org.jasig.portal.stats.events.PageRenderedEvent;
import org.jasig.portal.stats.events.PortalEvent;
import org.jasig.portal.stats.events.PortletActionEvent;
import org.jasig.portal.stats.events.SessionCreatedEvent;
import org.jasig.portal.stats.events.SessionDestroyedEvent;
import org.jasig.portal.stats.events.UnrecognizedEventTypeException;
import org.jasig.portal.stats.item.read.SessionGroupsLoader;
import org.jasig.portal.stats.om.ChannelInfo;
import org.jasig.portal.stats.om.TabInfo;
import org.springframework.jdbc.core.RowMapper;

/**
 * Maps a database row into a PortalEvent
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class PortalEventMapper implements RowMapper {
    private ChannelInfoLoader channelInfoLoader;
    private TabInfoLoader tabInfoLoader;
    private SessionGroupsLoader sessionGroupsLoader;
    
    /**
     * @return the channelInfoLoader
     */
    public ChannelInfoLoader getChannelInfoLoader() {
        return channelInfoLoader;
    }
    /**
     * @param channelInfoLoader the channelInfoLoader to set
     */
    public void setChannelInfoLoader(ChannelInfoLoader channelInfoLoader) {
        this.channelInfoLoader = channelInfoLoader;
    }

    /**
     * @return the tabInfoLoader
     */
    public TabInfoLoader getTabInfoLoader() {
        return tabInfoLoader;
    }
    /**
     * @param tabInfoLoader the tabInfoLoader to set
     */
    public void setTabInfoLoader(TabInfoLoader tabInfoLoader) {
        this.tabInfoLoader = tabInfoLoader;
    }

    /**
     * @return the sessionGroupsLoader
     */
    public SessionGroupsLoader getSessionGroupsLoader() {
        return sessionGroupsLoader;
    }
    /**
     * @param sessionGroupsLoader the sessionGroupsLoader to set
     */
    public void setSessionGroupsLoader(SessionGroupsLoader sessionGroupsLoader) {
        this.sessionGroupsLoader = sessionGroupsLoader;
    }


    /* (non-Javadoc)
     * @see org.springframework.jdbc.core.RowMapper#mapRow(java.sql.ResultSet, int)
     */
    public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
        final String eventTypeName = rs.getString("EVENT_TYPE");
        final EventType eventType;
        try {
            eventType = EventType.valueOf(eventTypeName);
        }
        catch (Exception e) {
            throw new UnrecognizedEventTypeException("Could not determine EventType for EVENT_TYPE of '" + eventTypeName + "'", e);
        }
        
        final long eventId = rs.getLong("EVENT_ID");
        final String userName = rs.getString("USERID");
        final Date activityDate = new Date(rs.getTimestamp("ACT_DATE").getTime());
        final long sessionId = rs.getLong("SESSION_ID");
        
        final PortalEvent portalEvent;
        switch (eventType) {
            case SESSION_CREATED: {
                portalEvent = new SessionCreatedEvent(eventId);
            }
            break;

            case SESSION_DESTROYED: {
                portalEvent = new SessionDestroyedEvent(eventId);
            }
            break;
            
            case CHANNEL_RENDERED:
            case CHANNEL_TARGETED:
            case PORTLET_ACTION: {
                portalEvent = this.createChannelEvent(eventId, eventType, rs);
            } break;

            case PAGE_RENDER_TIME: {
                final PageRenderedEvent pageRenderedEvent = new PageRenderedEvent(eventId);
                
                final int profileId = rs.getInt("PROFILE_ID");
                final String layoutStructId = rs.getString("FOLDER_ID");
                final TabInfo tabInfo = this.tabInfoLoader.getTabInfo(userName, profileId, layoutStructId);
                pageRenderedEvent.setTabInfo(tabInfo);
                
                final long renderTime = rs.getLong("RENDER_TIME");
                pageRenderedEvent.setTime(renderTime);

                portalEvent = pageRenderedEvent;
            }
            break;

            default: {
                throw new UnrecognizedEventTypeException("No handling configured for EventType: " + eventType); 
            }
        }
        
        final Map<String, Integer> groupKeys = this.sessionGroupsLoader.getGroupKeys(sessionId);
        portalEvent.setGroups(groupKeys);
        
        portalEvent.setUserName(userName);
        portalEvent.setActivityDate(activityDate);
        
        return portalEvent;
    }

    protected ChannelEvent createChannelEvent(final long eventId, final EventType eventType, ResultSet rs) throws SQLException {
        final ChannelEvent channelEvent;
        switch (eventType) {
            case CHANNEL_RENDERED: {
                final ChannelRenderedEvent channelRenderedEvent = new ChannelRenderedEvent(eventId);
                
                final long renderTime = rs.getLong("RENDER_TIME");
                channelRenderedEvent.setTime(renderTime);
                
                final String cachedFlag = rs.getString("RENDER_CACHED");
                channelRenderedEvent.setCached("Y".equals(cachedFlag));

                channelEvent = channelRenderedEvent;
            }
            break;

            case CHANNEL_TARGETED: {
                channelEvent = new ChannelTargetedEvent(eventId);
            }
            break;

            case PORTLET_ACTION: {
                final PortletActionEvent portletActionEvent = new PortletActionEvent(eventId);
                
                final long renderTime = rs.getLong("RENDER_TIME");
                portletActionEvent.setTime(renderTime);
                
                channelEvent = portletActionEvent;
            }
            break;

            default: {
                throw new UnrecognizedEventTypeException("No handling configured for EventType: " + eventType); 
            }
        }

        final long channelDefinitionId = rs.getLong("CHAN_DEF_ID");
        final ChannelInfo channelInfo = this.channelInfoLoader.getChannelInfo(channelDefinitionId);
        channelEvent.setChannelInfo(channelInfo);

        return channelEvent;
    }
}
