package org.jasig.portal.stats.om;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.springframework.util.Assert;

/**
 * The start and end dates for a time interval. The start date is inclusive, the end date is exclusive. Also includes
 * the synthetic keys for the start date and start time of the interval
 */
public class IntervalInfo implements Serializable {
    private static final long serialVersionUID = 1L;

    private final Date start;
    private final Date end;
    private final int startDateKey;
    private final int startTimeKey;
    private int timeSpan = -1;

    public IntervalInfo(Date start, Date end, int startDateKey, int startTimeKey) {
        Assert.notNull(start, "start can not be null");
        Assert.notNull(end, "end can not be null");
        if (!start.before(end)) {
            throw new IllegalArgumentException("Start date must be before end date. start=" + start + ", end=" + end);
        }
        
        this.start = start;
        this.end = end;
        this.startDateKey = startDateKey;
        this.startTimeKey = startTimeKey;
    }

    /**
     * @return The first date in the interval, inclusive
     */
    public Date getStart() {
        return this.start;
    }
    /**
     * @return The last date in the interval, exclusive
     */
    public Date getEnd() {
        return this.end;
    }
    /**
     * @return The database date fact key for interval
     */
    public int getStartDateKey() {
        return this.startDateKey;
    }
    /**
     * @return The database time fact key for the interval
     */
    public int getStartTimeKey() {
        return this.startTimeKey;
    }
    /**
     * @return Minutes between start and end
     */
    public int getTimeSpan() {
        if (this.timeSpan < 0) {
            this.timeSpan = (int)((this.end.getTime() - this.start.getTime()) / (60l * 1000l));
        }
        return this.timeSpan;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    @Override
    public boolean equals(Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IntervalInfo)) {
            return false;
        }
        IntervalInfo rhs = (IntervalInfo) object;
        return new EqualsBuilder()
            .append(this.startDateKey, rhs.startDateKey)
            .append(this.startTimeKey, rhs.startTimeKey)
            .append(this.start, rhs.start)
            .append(this.end, rhs.end)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return new HashCodeBuilder(-2116278315, 401037003)
            .append(this.startDateKey)
            .append(this.startTimeKey)
            .append(this.start)
            .append(this.end)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
            .append("start", this.start)
            .append("startDateKey", this.startDateKey)
            .append("startTimeKey", this.startTimeKey)
            .append("end", this.end)
            .append("timeSpan", this.getTimeSpan())
            .toString();
    }
}