/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.purge;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import org.jasig.portal.stats.Average;
import org.jasig.portal.stats.SqlLoader;
import org.jasig.portal.stats.shutdown.ShutdownListener;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.jdbc.core.RowMapperResultSetExtractor;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcOperations;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedSingleColumnRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.jdbc.support.JdbcUtils;
import org.springframework.jdbc.support.nativejdbc.NativeJdbcExtractor;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * @author Eric Dalquist
 * @version $Revision: 46445 $
 */
public class StatsPurgingDaoImpl extends SimpleJdbcDaoSupport implements StatsPurgingDao, ShutdownListener {
    private final static DecimalFormat AVG_FORMATTER = new DecimalFormat("0.00");
    
    private final AtomicLong executing = new AtomicLong(0);
    private final AtomicBoolean shutdown = new AtomicBoolean(false);
    private final Set<Statement> currentStatements = Collections.synchronizedSet(new HashSet<Statement>());
    
    private final ParameterizedRowMapper<Long> mapper;
    
    private final String dateParam;
    private final String sessionIdParam;
    private final String eventLimitParam;
    
    private final String selectEventsToPurge;
    private final String deleteEventsForSession;
    private final String deleteSessionGroups;
    private final String deleteSession;

    private TransactionTemplate transactionTemplate;
    private PlatformTransactionManager transactionManager;
    private int sessionBatchSize = 25;
    private int eventsPerSessionRatio = 25;
    private int sessionLoadSize = 5000;


    public PlatformTransactionManager getTransactionManager() {
        return this.transactionManager;
    }
    /**
     * The transaction manager to use for wrapping delete calls in for batching
     */
    public void setTransactionManager(PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public int getSessionBatchSize() {
        return this.sessionBatchSize;
    }
    /**
     * Number of sessions to purge in a single transaction, defaults to 25
     */
    public void setSessionBatchSize(int sessionBatchSize) {
        this.sessionBatchSize = sessionBatchSize;
    }
    
    public int getEventsPerSessionRatio() {
        return eventsPerSessionRatio;
    }
    /**
     * Ratio of number events per session, defaults to 25
     */
    public void setEventsPerSessionRatio(int eventsPerSessionRatio) {
        this.eventsPerSessionRatio = eventsPerSessionRatio;
    }

    public int getSessionLoadSize() {
        return sessionLoadSize;
    }
    /**
     * Number of sessions to load in a single driving query, defaults to 5000
     */
    public void setSessionLoadSize(int sessionLoadSize) {
        this.sessionLoadSize = sessionLoadSize;
    }
    
    
    public StatsPurgingDaoImpl() {
        final SqlLoader sqlLoader = new SqlLoader("org/jasig/portal/stats/purge/StatsPurgingDaoImplSql.xml", this.getClass().getClassLoader());
        this.dateParam = sqlLoader.getSql("dateParam");
        this.sessionIdParam = sqlLoader.getSql("sessionIdParam");
        this.eventLimitParam = sqlLoader.getSql("eventLimitParam");
        
        this.selectEventsToPurge = sqlLoader.getSql("selectEventsToPurge");
        this.deleteEventsForSession = sqlLoader.getSql("deleteEventsForSession");
        this.deleteSessionGroups = sqlLoader.getSql("deleteSessionGroups");
        this.deleteSession = sqlLoader.getSql("deleteSession");
        
        this.mapper = ParameterizedSingleColumnRowMapper.newInstance(Long.class);
    }
    
    /* (non-Javadoc)
     * @see org.springframework.dao.support.DaoSupport#initDao()
     */
    @Override
    protected void initDao() throws Exception {
        this.transactionTemplate = new TransactionTemplate(this.transactionManager);
        this.transactionTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
        this.transactionTemplate.afterPropertiesSet();
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.purge.StatsPurgingDao#purgeStatsBefore(java.util.Date)
     */
    @SuppressWarnings("unchecked")
    public int purgeStatsBefore(Date cutoff) {
        this.executing.incrementAndGet();
        try {
            if (this.shutdown.get()) {
                return 0;
            }
        
            final SimpleJdbcTemplate simpleJdbcTemplate = this.getSimpleJdbcTemplate();
            final NamedParameterJdbcOperations namedJdbcOperations = simpleJdbcTemplate.getNamedParameterJdbcOperations();
            
            final Map<String, Object> sessionIdQueryParams = new HashMap<String, Object>(); 
            sessionIdQueryParams.put(this.dateParam, cutoff);
            sessionIdQueryParams.put(this.eventLimitParam, this.sessionLoadSize * this.eventsPerSessionRatio);
            
            if (logger.isInfoEnabled()) {
                logger.info("Loading up to " + sessionIdQueryParams.get(this.eventLimitParam) + " events to be purged before " + cutoff);
            }
            
            final long queryStart = System.currentTimeMillis();
            
            final List<Long> sessionIds = (List<Long>)namedJdbcOperations.execute(this.selectEventsToPurge, sessionIdQueryParams, new PreparedStatementCallback() {
                public Object doInPreparedStatement(PreparedStatement ps) throws SQLException, DataAccessException {
                    ResultSet rs = null;
                    currentStatements.add(ps);
                    try {
                        if (shutdown.get()) {
                            return Collections.EMPTY_LIST;
                        }
                        
                        rs = ps.executeQuery();
                        
                        ResultSet rsToUse = rs;
                        final NativeJdbcExtractor nativeJdbcExtractor = getJdbcTemplate().getNativeJdbcExtractor();
                        if (nativeJdbcExtractor != null) {
                            rsToUse = nativeJdbcExtractor.getNativeResultSet(rs);
                        }
                        
                        final RowMapperResultSetExtractor rse = new RowMapperResultSetExtractor(mapper);
                        return rse.extractData(rsToUse);
                    }
                    finally {
                        currentStatements.remove(ps);
                        JdbcUtils.closeResultSet(rs);
                    }
                }
            });
            
            
            final long queryTime = System.currentTimeMillis() - queryStart;
            if (logger.isInfoEnabled()) {
                logger.info("Loaded " + sessionIds.size() + " sessions to be purged in " + queryTime + "ms");
            }
            
            final List<Map<String, Object>> deleteParamBuilder = new ArrayList<Map<String,Object>>(this.sessionBatchSize);
            int purgedSessions = 0;
            final Average deleteAverage = new Average();
            
            final long purgeStart = System.currentTimeMillis();
            for (final Iterator<Long> sessionIdItr = sessionIds.iterator(); sessionIdItr.hasNext() && !this.shutdown.get(); ) {
                deleteParamBuilder.clear();
                
                //Build batch delete parameters list
                while (sessionIdItr.hasNext() && deleteParamBuilder.size() < this.sessionBatchSize) {
                    final Long sessionId = sessionIdItr.next();
                    final Map<String, Object> params = Collections.singletonMap(this.sessionIdParam, (Object)sessionId);
                    deleteParamBuilder.add(params);
                }
    
                //Delete the events & session data in a transaction
                final Map<?, ?>[] deleteParams = deleteParamBuilder.toArray(new Map[deleteParamBuilder.size()]);
                final PurgeSessionsCallback purgeSessionsCallback = new PurgeSessionsCallback(deleteParams);
                
                final long deleteStart = System.currentTimeMillis();
                this.transactionTemplate.execute(purgeSessionsCallback);
                final long deleteTime = System.currentTimeMillis() - deleteStart;

                if (logger.isDebugEnabled()) {
                    logger.debug("Purged " + deleteParams.length + " sessions  in " + deleteTime + "ms");
                }
                
                purgedSessions += deleteParamBuilder.size();
                deleteAverage.add(deleteTime);
            }
            final long purgeTime = System.currentTimeMillis() - purgeStart;
            
            if (logger.isInfoEnabled()) {
                final double avgSessionTime = ((double)purgeTime) / ((double) purgedSessions);
                logger.info("Purged " + purgedSessions + " sessions in " + purgeTime + "ms, averaged " + AVG_FORMATTER.format(deleteAverage.getAverage()) + "ms per transaction and " + AVG_FORMATTER.format(avgSessionTime) + "ms per session");
            }
        
            return purgedSessions;
        }
        finally {
            this.executing.decrementAndGet();
            
            //If shutdown notify all waiting threads
            if (this.shutdown.get()) {
                synchronized (this.shutdown) {
                    this.shutdown.notifyAll();
                }
            }
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.shutdown.ShutdownListener#shutdown()
     */
    public void shutdown() {
        this.shutdown.getAndSet(true);
        
        //If any threads are still executing wait
        synchronized (this.shutdown) {
            while (this.executing.get() > 0) {
                //Cancel any currently running statement
                synchronized (this.currentStatements) {
                    for (final Iterator<Statement> statementItr = this.currentStatements.iterator(); statementItr.hasNext(); ) {
                        final Statement statement = statementItr.next();
                        try {
                            statement.cancel();
                        }
                        catch (SQLException e) {
                            this.logger.warn("Failed when trying to cancel currently running statement", e);
                        }
                        statementItr.remove();
                    }
                }
                
                try {
                    this.shutdown.wait();
                }
                catch (InterruptedException e) {
                    this.logger.warn("Interrupted while waiting for shutdown notification");
                }
            }
        }
    }
    
    private final class PurgeSessionsCallback implements TransactionCallback {
        private final Map<?, ?>[] deleteParams;

        private PurgeSessionsCallback(Map<?, ?>[] deleteParamBuilder) {
            this.deleteParams = deleteParamBuilder;
        }

        /* (non-Javadoc)
         * @see org.springframework.transaction.support.TransactionCallback#doInTransaction(org.springframework.transaction.TransactionStatus)
         */
        public Object doInTransaction(TransactionStatus status) {
            final SimpleJdbcTemplate simpleJdbcTemplate = getSimpleJdbcTemplate();

            if (logger.isDebugEnabled()) {
                logger.debug("Puring events for sessions: " + Arrays.asList(this.deleteParams));
            }
   
            simpleJdbcTemplate.batchUpdate(deleteEventsForSession, this.deleteParams);
            simpleJdbcTemplate.batchUpdate(deleteSessionGroups, this.deleteParams);
            simpleJdbcTemplate.batchUpdate(deleteSession, this.deleteParams);
   
            return null;
        }
    }
}
