/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.purge;

import java.util.Date;

import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.stats.dao.JobDateStore;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.step.AbstractStep;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.repeat.ExitStatus;

/**
 * Step that uses the last job event DAO to retrieve the date of the last event processed and subtracts the
 * configured number of days from that date and calls {@link StatsPurgingDao#purgeStatsBefore(Date)} using
 * that calculated value.
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class StatsPurgingStep extends AbstractStep {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private JobDateStore jobDateStore;
    private StatsPurgingDao statsPurgingDao;
    private String targetJobName;
    private int daysToKeep = 14;
    
    public int getDaysToKeep() {
        return daysToKeep;
    }
    /**
     * The number of days before the lastEvent date to keep, defaults to 14
     */
    public void setDaysToKeep(int daysToKeep) {
        this.daysToKeep = daysToKeep;
    }
    
    public StatsPurgingDao getStatsPurgingDao() {
        return this.statsPurgingDao;
    }
    /**
     * The DAO to use for purging old stats
     */
    public void setStatsPurgingDao(StatsPurgingDao statsPurgingDao) {
        this.statsPurgingDao = statsPurgingDao;
    }
    
    public JobDateStore getLastJobEventDao() {
        return this.jobDateStore;
    }
    /**
     * DAO to retrieve last processed event timestamp from
     */
    public void setLastJobEventDao(JobDateStore jobDateStore) {
        this.jobDateStore = jobDateStore;
    }

    public String getTargetJobName() {
        return this.targetJobName;
    }
    /**
     * Name of the job to get the last processed timestamp for
     */
    public void setTargetJobName(String targetJobName) {
        this.targetJobName = targetJobName;
    }
    
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.step.AbstractStep#doExecute(org.springframework.batch.core.StepExecution)
     */
    @Override
    protected ExitStatus doExecute(StepExecution stepExecution) throws Exception {
        final Date lastEvent = this.jobDateStore.getStoredDate(this.targetJobName);

        if (lastEvent == null) {
            this.logger.info("No lastEvent date for job '" + this.targetJobName + "', nothing will happen");
            return ExitStatus.NOOP;
        }

        final Date purgeDate = DateUtils.addDays(lastEvent, -this.daysToKeep);
        this.logger.info("Found lastEvent=" + lastEvent + ", will delete events before=" + purgeDate);
        this.statsPurgingDao.purgeStatsBefore(purgeDate);
        
        return ExitStatus.FINISHED;
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.step.AbstractStep#close(org.springframework.batch.item.ExecutionContext)
     */
    @Override
    protected void close(ExecutionContext ctx) throws Exception {
        //Ignore
    }
    /* (non-Javadoc)
     * @see org.springframework.batch.core.step.AbstractStep#open(org.springframework.batch.item.ExecutionContext)
     */
    @Override
    protected void open(ExecutionContext ctx) throws Exception {
        //Ignore
    }
}
