/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.shutdown;

import java.util.HashSet;
import java.util.Set;

import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.springframework.batch.core.BatchStatus;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobExecutionListener;
import org.springframework.beans.factory.InitializingBean;


/**
 * Subclass of ShutdownHandler that first sets the quartz scheduler to standby then waits for all Jobs to
 * complete execution before allowing the shutdown to complete.
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class JobShutdownHandler extends ShutdownHandler implements InitializingBean, JobExecutionListener {
    final Set<JobExecution> runningJobs = new HashSet<JobExecution>();
    
    Scheduler jobScheduler;
    public Scheduler getJobScheduler() {
        return this.jobScheduler;
    }
    /**
     * Control Scheduler for the Job launching scheduler
     */
    public void setJobScheduler(Scheduler jobScheduler) {
        this.jobScheduler = jobScheduler;
    }
    

    /* (non-Javadoc)
     * @see org.jasig.portal.stats.shutdown.ShutdownHandler#preShutdownListeners()
     */
    @Override
    protected void preShutdownListeners() {
        //stop the scheduler
        this.logger.info("Setting scheduler to standby");
        try {
            this.jobScheduler.standby();
        }
        catch (SchedulerException se) {
            this.logger.warn("Exception while setting standby on scheduler", se);
        }
        this.logger.info("Scheduler should now be in standby");
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.stats.shutdown.ShutdownHandler#postShutdownListeners()
     */
    @Override
    protected void postShutdownListeners() {
        //Make a copy of the running jobs Set to avoid potential deadlock issues
        final JobExecution[] runningJobsCopy;
        synchronized (this.runningJobs) {
            runningJobsCopy = this.runningJobs.toArray(new JobExecution[this.runningJobs.size()]);
        }
        
        //Wait for any currently running jobs to complete
        for (final JobExecution currentExecution : runningJobsCopy) {
            this.logger.info("Waiting for JobExecution: " + currentExecution + " to stop");
            while (!BatchStatus.STOPPED.equals(currentExecution.getStatus()) &&
                    !BatchStatus.COMPLETED.equals(currentExecution.getStatus()) &&
                    !BatchStatus.FAILED.equals(currentExecution.getStatus())) {

                Thread.yield();
            }
            this.logger.info("JobExecution: " + currentExecution + " has stopped");
        }
    }

    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#afterJob(org.springframework.batch.core.JobExecution)
     */
    public void afterJob(JobExecution jobExecution) {
        synchronized (this.runningJobs) {
            this.runningJobs.remove(jobExecution);
        }
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#beforeJob(org.springframework.batch.core.JobExecution)
     */
    public void beforeJob(JobExecution jobExecution) {
        synchronized (this.runningJobs) {
            this.runningJobs.add(jobExecution);
        }
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#onError(org.springframework.batch.core.JobExecution, java.lang.Throwable)
     */
    public void onError(JobExecution jobExecution, Throwable e) {
        synchronized (this.runningJobs) {
            this.runningJobs.remove(jobExecution);
        }
    }
    
    /* (non-Javadoc)
     * @see org.springframework.batch.core.JobExecutionListener#onInterrupt(org.springframework.batch.core.JobExecution)
     */
    public void onInterrupt(JobExecution jobExecution) {
        synchronized (this.runningJobs) {
            this.runningJobs.remove(jobExecution);
        }
    }
}
