/**
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.stats.shutdown;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;

/**
 * Provides a shutdown socket service for an application. When the class is created and {@link #afterPropertiesSet()} is called
 * a socket is opened and monitored for shutdown requests. When a shutdown request occurs registered shutdown listeners are called
 * along with pre and post shutdown listener local methods (for subclassing). The {@link #waitForShutdown()} is provided to allow
 * application code, usually the main thread, to wait for a shutdown request. All APIs provided are thread-safe, locking is done
 * within {@link #shutdown()} and {@link #waitForShutdown()} on a single lock so provided {@link ShutdownListener}s should be careful
 * to never wait for shutdown when being notified of a shutdown occurring. 
 * 
 * @author Eric Dalquist
 * @version $Revision: 46014 $
 */
public class ShutdownHandler implements InitializingBean {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private final Lock shutdownLock = new ReentrantLock();
    private final Condition shutdownCond = this.shutdownLock.newCondition();
    private final AtomicBoolean shutdownRequested = new AtomicBoolean(false);
    private final AtomicBoolean shutdownComplete = new AtomicBoolean(false);
    
    private Thread shutdownSocketThread = null;
    private ServerSocket shutdownSocket = null;
    
    private String host = "localhost";
    private int port = 7456;
    private String cmdShutdownNoWait = "SHUTDOWN_NO_WAIT";
    private String cmdShutdownWait = "SHUTDOWN_AND_WAIT";
    private String cmdStatus = "STATUS";
    private List<ShutdownListener> shutdownListeners;

    
    public String getHost() {
        return this.host;
    }
    /**
     * Host to listen on for shutdown requests
     */
    public void setHost(String host) {
        this.host = host;
    }

    public int getPort() {
        return this.port;
    }

    /**
     * Port to listen on for shutdown requests
     */
    public void setPort(int port) {
        this.port = port;
    }

    public String getCmdShutdownNoWait() {
        return this.cmdShutdownNoWait;
    }
    /**
     * Command string to be sent to the shutdown socket, the connection will be closed immediately after receiving the command 
     */
    public void setCmdShutdownNoWait(String command) {
        this.cmdShutdownNoWait = command;
    }
    
    public String getCmdShutdownWait() {
        return this.cmdShutdownWait;
    }
    /**
     * Command string to be sent to the shutdown socket, the connection will be held open until shutdown is complete
     */
    public void setCmdShutdownWait(String cmdShutdownWait) {
        this.cmdShutdownWait = cmdShutdownWait;
    }

    public String getCmdStatus() {
        return this.cmdStatus;
    }
    /**
     * Command string to be sent to the shutdown socket to dermine the applications state (running or shutting down)
     */
    public void setCmdStatus(String cmdStatus) {
        this.cmdStatus = cmdStatus;
    }
    
    public List<ShutdownListener> getShutdownListeners() {
        return this.shutdownListeners;
    }
    /**
     * Listeners that want to be notified of the shutdown
     */
    public void setShutdownListeners(List<ShutdownListener> shutdownListeners) {
        this.shutdownListeners = shutdownListeners;
    }

    public void afterPropertiesSet() throws Exception {
        final InetAddress bindHost;
        try {
            bindHost = InetAddress.getByName(this.host);
        }
        catch (UnknownHostException uhe) {
            throw new RuntimeException("Failed to create InetAddress for host '" + host  + "'", uhe);
        }
        
        try {
            this.shutdownSocket = new ServerSocket(this.port, 10, bindHost);
        }
        catch (IOException ioe) {
            throw new RuntimeException("Failed to create shutdown socket on '" + bindHost + "' and " + this.port, ioe);
        }
        
        this.logger.info("Bound shutdown socket to " + bindHost + ":" + this.port + " Starting listener thread for shutdown requests.");
        
        this.shutdownSocketThread = new Thread(new ShutdownSocketListener(), "ShutdownListener-" + this.host + ":" + this.port);
        this.shutdownSocketThread.setDaemon(true);
        this.shutdownSocketThread.start();
        
        //Register a shutdown handler
        Runtime.getRuntime().addShutdownHook(new Thread(new ShutdownHookHandler(), "JVM Shutdown Hook"));
        this.logger.debug("Registered JVM shutdown hook");
    }

    /**
     * If shutdown isn't complete will wait on the shutdown lock for shutdown to complete.
     * DOES NOT TRIGGER SHUTDOWN
     */
    public final void waitForShutdown() {
        if (this.shutdownComplete.get()) {
            return;
        }

        this.shutdownLock.lock();
        try {
            try {
                this.shutdownCond.await();
            }
            catch (InterruptedException e) {
                this.logger.warn("Interrupted waiting for shutdown condition", e);
            }
        }
        finally {
            this.shutdownLock.unlock();
        }
    }
    
    /**
     * Called before the shutdown listeners, happens inside the shutdown lock
     */
    protected void preShutdownListeners() {
    }
    
    /**
     * Called after the shutdown listeners, happens inside the shutdown lock
     */
    protected void postShutdownListeners() {
    }

    /**
     * Calls shutdown hooks and cleans up shutdown listener code, notifies all waiting threads on completion
     */
    public final void shutdown() {
        final boolean shuttingDown = this.shutdownRequested.getAndSet(true);
        if (shuttingDown) {
            logger.info("Already shutting down, ignoring duplicate request");
            return;
        }
    
        //Grab the shutdown lock and set the shutdown flags
        this.shutdownLock.lock();
        try {
            this.preShutdownListeners();
            
            //Run other shutdown tasks
            if (this.shutdownListeners != null) {
                for (final ShutdownListener shutdownListener : this.shutdownListeners) {
                    try {
                        this.logger.info("Calling ShutdownListener: " + shutdownListener);
                        shutdownListener.shutdown();
                        this.logger.info("ShutdownListener: " + shutdownListener + " complete");
                    }
                    catch (Exception e) {
                        this.logger.warn("ShutdownListener " + shutdownListener + " threw an exception", e);
                    }
                }
            }
            
            this.postShutdownListeners();
            
            //Shutdown the listening socket
            if (!this.shutdownSocket.isClosed()) {
                try {
                    this.shutdownSocket.close();
                }
                catch (IOException e) {
                    //ignore
                }
            }
            
            this.shutdownComplete.set(true);
            this.shutdownCond.signalAll();
        }
        finally {
            this.shutdownLock.unlock();
        }
    }


    /**
     * Runable for waiting on connections to the shutdown socket and handling them
     */
    private class ShutdownSocketListener implements Runnable {
        /* (non-Javadoc)
         * @see java.lang.Runnable#run()
         */
        public void run() {
            try {
                while (!shutdownRequested.get() && !shutdownSocket.isClosed()) {
                    try {
                        final Socket connection = shutdownSocket.accept();
                        final ShutdownSocketHandler shutdownSocketHandler = new ShutdownSocketHandler(connection);
                        final Thread shutdownRequestThread = new Thread(shutdownSocketHandler, "ShutdownHandler-" + connection.getInetAddress() + ":" + connection.getPort());
                        shutdownRequestThread.setDaemon(true);
                        shutdownRequestThread.start();
                    }
                    catch (SocketException se) {
                        if (shutdownSocket.isClosed()) {
                            logger.info("Caught SocketException on shutdownSocket, assuming close() was called: " + se);
                        }
                        else {
                            logger.warn("Exception while handling connection to shutdown socket, ignoring", se);
                        }
                    }
                    catch (IOException ioe) {
                        logger.warn("Exception while handling connection to shutdown socket, ignoring", ioe);
                    }
                }
            }
            finally {
                if (!shutdownSocket.isClosed()) {
                    try {
                        shutdownSocket.close();
                    }
                    catch (IOException ioe) {
                        //Ignore
                    }
                }
            }
        }
    }
    
    /**
     * Runnable to handle connections to the shutdown socket
     */
    private class ShutdownSocketHandler implements Runnable {
        private final Socket shutdownConnection;
        
        public ShutdownSocketHandler(Socket shutdownConnection) {
            this.shutdownConnection = shutdownConnection;
        }

        /* (non-Javadoc)
         * @see java.lang.Runnable#run()
         */
        public void run() {
            boolean shutdown = false;

            try {
                final BufferedReader reader = new BufferedReader(new InputStreamReader(shutdownConnection.getInputStream()));
                final PrintWriter writer = new PrintWriter(this.shutdownConnection.getOutputStream());
                try {
                    final String recievedCommand = reader.readLine();
                
                    if (cmdShutdownWait.equals(recievedCommand)) {
                        logger.info("Recieved request for shutdown");
                        writer.println(new Date() + ": Starting Shutdown and waiting");
                        writer.flush();
                        shutdown();
                        writer.println(new Date() + ": Shutdown Complete");
                    }
                    else if (cmdShutdownNoWait.equals(recievedCommand)) {
                        logger.info("Recieved request for shutdown");
                        writer.println(new Date() + ": Starting Shutdown and disconnecting shutdown socket");
                        shutdown = true;
                    }
                    else if (cmdStatus.equals(recievedCommand)) {
                        logger.debug("Recieved request for status");
                        if (shutdownRequested.get()) {
                            writer.println(new Date() + ": Shutting down");
                        }
                        else {
                            writer.println(new Date() + ": Running");
                        }
                    }
                    else {
                        writer.println(new Date() + ": Unknown command '" + recievedCommand + "'");
                    }
                }
                finally {
                    writer.flush();
                    IOUtils.closeQuietly(reader);
                    IOUtils.closeQuietly(writer);
                }
            }
            catch (IOException e) {
                logger.warn("Exception while hanlding connection to shutdown socket, ignoring", e);
            }
            finally {
                if (shutdownConnection != null) {
                    try {
                        shutdownConnection.close();
                    }
                    catch (IOException ioe) {
                        //Ignore
                    }
                }
                
                //To handle shutdown-no-wait calls
                if (shutdown) {
                    shutdown();
                }
            }
        }
    }

    /**
     * Runnable that calls shutdown, used for JVM shutdown hook
     */
    private class ShutdownHookHandler implements Runnable {
        /* (non-Javadoc)
         * @see java.lang.Runnable#run()
         */
        public void run() {
            logger.info("JVM shutdown hook called");
            shutdown();
        }
    }
}